# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from copy import deepcopy
import sys
from typing import Any, Optional, TYPE_CHECKING, cast

from azure.core.pipeline import policies
from azure.core.rest import HttpRequest, HttpResponse
from azure.core.settings import settings
from azure.mgmt.core import ARMPipelineClient
from azure.mgmt.core.policies import ARMAutoResourceProviderRegistrationPolicy
from azure.mgmt.core.tools import get_arm_endpoints

from ._configuration import NetAppManagementClientConfiguration
from ._utils.serialization import Deserializer, Serializer
from .operations import (
    AccountsOperations,
    ActiveDirectoryConfigsOperations,
    BackupPoliciesOperations,
    BackupVaultsOperations,
    BackupsOperations,
    BackupsUnderAccountOperations,
    BackupsUnderBackupVaultOperations,
    BackupsUnderVolumeOperations,
    BucketsOperations,
    CachesOperations,
    ElasticAccountsOperations,
    ElasticBackupPoliciesOperations,
    ElasticBackupVaultsOperations,
    ElasticBackupsOperations,
    ElasticCapacityPoolsOperations,
    ElasticSnapshotPoliciesOperations,
    ElasticSnapshotsOperations,
    ElasticVolumesOperations,
    NetAppResourceOperations,
    NetAppResourceQuotaLimitsAccountOperations,
    NetAppResourceQuotaLimitsOperations,
    NetAppResourceRegionInfosOperations,
    NetAppResourceUsagesOperations,
    Operations,
    PoolsOperations,
    RansomwareReportsOperations,
    SnapshotPoliciesOperations,
    SnapshotsOperations,
    SubvolumesOperations,
    VolumeGroupsOperations,
    VolumeQuotaRulesOperations,
    VolumesOperations,
)

if sys.version_info >= (3, 11):
    from typing import Self
else:
    from typing_extensions import Self  # type: ignore

if TYPE_CHECKING:
    from azure.core import AzureClouds
    from azure.core.credentials import TokenCredential


class NetAppManagementClient:  # pylint: disable=too-many-instance-attributes
    """Microsoft NetApp Files Azure Resource Provider specification.

    :ivar operations: Operations operations
    :vartype operations: azure.mgmt.netapp.operations.Operations
    :ivar net_app_resource_quota_limits_account: NetAppResourceQuotaLimitsAccountOperations
     operations
    :vartype net_app_resource_quota_limits_account:
     azure.mgmt.netapp.operations.NetAppResourceQuotaLimitsAccountOperations
    :ivar volume_groups: VolumeGroupsOperations operations
    :vartype volume_groups: azure.mgmt.netapp.operations.VolumeGroupsOperations
    :ivar backups: BackupsOperations operations
    :vartype backups: azure.mgmt.netapp.operations.BackupsOperations
    :ivar volumes: VolumesOperations operations
    :vartype volumes: azure.mgmt.netapp.operations.VolumesOperations
    :ivar snapshots: SnapshotsOperations operations
    :vartype snapshots: azure.mgmt.netapp.operations.SnapshotsOperations
    :ivar snapshot_policies: SnapshotPoliciesOperations operations
    :vartype snapshot_policies: azure.mgmt.netapp.operations.SnapshotPoliciesOperations
    :ivar backup_policies: BackupPoliciesOperations operations
    :vartype backup_policies: azure.mgmt.netapp.operations.BackupPoliciesOperations
    :ivar volume_quota_rules: VolumeQuotaRulesOperations operations
    :vartype volume_quota_rules: azure.mgmt.netapp.operations.VolumeQuotaRulesOperations
    :ivar ransomware_reports: RansomwareReportsOperations operations
    :vartype ransomware_reports: azure.mgmt.netapp.operations.RansomwareReportsOperations
    :ivar backup_vaults: BackupVaultsOperations operations
    :vartype backup_vaults: azure.mgmt.netapp.operations.BackupVaultsOperations
    :ivar buckets: BucketsOperations operations
    :vartype buckets: azure.mgmt.netapp.operations.BucketsOperations
    :ivar caches: CachesOperations operations
    :vartype caches: azure.mgmt.netapp.operations.CachesOperations
    :ivar elastic_accounts: ElasticAccountsOperations operations
    :vartype elastic_accounts: azure.mgmt.netapp.operations.ElasticAccountsOperations
    :ivar elastic_capacity_pools: ElasticCapacityPoolsOperations operations
    :vartype elastic_capacity_pools: azure.mgmt.netapp.operations.ElasticCapacityPoolsOperations
    :ivar elastic_volumes: ElasticVolumesOperations operations
    :vartype elastic_volumes: azure.mgmt.netapp.operations.ElasticVolumesOperations
    :ivar elastic_snapshots: ElasticSnapshotsOperations operations
    :vartype elastic_snapshots: azure.mgmt.netapp.operations.ElasticSnapshotsOperations
    :ivar elastic_snapshot_policies: ElasticSnapshotPoliciesOperations operations
    :vartype elastic_snapshot_policies:
     azure.mgmt.netapp.operations.ElasticSnapshotPoliciesOperations
    :ivar elastic_backup_vaults: ElasticBackupVaultsOperations operations
    :vartype elastic_backup_vaults: azure.mgmt.netapp.operations.ElasticBackupVaultsOperations
    :ivar elastic_backup_policies: ElasticBackupPoliciesOperations operations
    :vartype elastic_backup_policies: azure.mgmt.netapp.operations.ElasticBackupPoliciesOperations
    :ivar elastic_backups: ElasticBackupsOperations operations
    :vartype elastic_backups: azure.mgmt.netapp.operations.ElasticBackupsOperations
    :ivar active_directory_configs: ActiveDirectoryConfigsOperations operations
    :vartype active_directory_configs:
     azure.mgmt.netapp.operations.ActiveDirectoryConfigsOperations
    :ivar net_app_resource_quota_limits: NetAppResourceQuotaLimitsOperations operations
    :vartype net_app_resource_quota_limits:
     azure.mgmt.netapp.operations.NetAppResourceQuotaLimitsOperations
    :ivar net_app_resource_region_infos: NetAppResourceRegionInfosOperations operations
    :vartype net_app_resource_region_infos:
     azure.mgmt.netapp.operations.NetAppResourceRegionInfosOperations
    :ivar accounts: AccountsOperations operations
    :vartype accounts: azure.mgmt.netapp.operations.AccountsOperations
    :ivar backups_under_account: BackupsUnderAccountOperations operations
    :vartype backups_under_account: azure.mgmt.netapp.operations.BackupsUnderAccountOperations
    :ivar pools: PoolsOperations operations
    :vartype pools: azure.mgmt.netapp.operations.PoolsOperations
    :ivar backups_under_backup_vault: BackupsUnderBackupVaultOperations operations
    :vartype backups_under_backup_vault:
     azure.mgmt.netapp.operations.BackupsUnderBackupVaultOperations
    :ivar backups_under_volume: BackupsUnderVolumeOperations operations
    :vartype backups_under_volume: azure.mgmt.netapp.operations.BackupsUnderVolumeOperations
    :ivar subvolumes: SubvolumesOperations operations
    :vartype subvolumes: azure.mgmt.netapp.operations.SubvolumesOperations
    :ivar net_app_resource: NetAppResourceOperations operations
    :vartype net_app_resource: azure.mgmt.netapp.operations.NetAppResourceOperations
    :ivar net_app_resource_usages: NetAppResourceUsagesOperations operations
    :vartype net_app_resource_usages: azure.mgmt.netapp.operations.NetAppResourceUsagesOperations
    :param credential: Credential used to authenticate requests to the service. Required.
    :type credential: ~azure.core.credentials.TokenCredential
    :param subscription_id: The ID of the target subscription. The value must be an UUID. Required.
    :type subscription_id: str
    :param base_url: Service host. Default value is None.
    :type base_url: str
    :keyword cloud_setting: The cloud setting for which to get the ARM endpoint. Default value is
     None.
    :paramtype cloud_setting: ~azure.core.AzureClouds
    :keyword api_version: The API version to use for this operation. Known values are
     "2026-04-15-preview" and None. Default value is None. If not set, the operation's default API
     version will be used. Note that overriding this default value may result in unsupported
     behavior.
    :paramtype api_version: str
    :keyword int polling_interval: Default waiting time between two polls for LRO operations if no
     Retry-After header is present.
    """

    def __init__(
        self,
        credential: "TokenCredential",
        subscription_id: str,
        base_url: Optional[str] = None,
        *,
        cloud_setting: Optional["AzureClouds"] = None,
        **kwargs: Any
    ) -> None:
        _endpoint = "{endpoint}"
        _cloud = cloud_setting or settings.current.azure_cloud  # type: ignore
        _endpoints = get_arm_endpoints(_cloud)
        if not base_url:
            base_url = _endpoints["resource_manager"]
        credential_scopes = kwargs.pop("credential_scopes", _endpoints["credential_scopes"])
        self._config = NetAppManagementClientConfiguration(
            credential=credential,
            subscription_id=subscription_id,
            base_url=cast(str, base_url),
            cloud_setting=cloud_setting,
            credential_scopes=credential_scopes,
            **kwargs
        )

        _policies = kwargs.pop("policies", None)
        if _policies is None:
            _policies = [
                policies.RequestIdPolicy(**kwargs),
                self._config.headers_policy,
                self._config.user_agent_policy,
                self._config.proxy_policy,
                policies.ContentDecodePolicy(**kwargs),
                ARMAutoResourceProviderRegistrationPolicy(),
                self._config.redirect_policy,
                self._config.retry_policy,
                self._config.authentication_policy,
                self._config.custom_hook_policy,
                self._config.logging_policy,
                policies.DistributedTracingPolicy(**kwargs),
                policies.SensitiveHeaderCleanupPolicy(**kwargs) if self._config.redirect_policy else None,
                self._config.http_logging_policy,
            ]
        self._client: ARMPipelineClient = ARMPipelineClient(base_url=cast(str, _endpoint), policies=_policies, **kwargs)

        self._serialize = Serializer()
        self._deserialize = Deserializer()
        self._serialize.client_side_validation = False
        self.operations = Operations(self._client, self._config, self._serialize, self._deserialize)
        self.net_app_resource_quota_limits_account = NetAppResourceQuotaLimitsAccountOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.volume_groups = VolumeGroupsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.backups = BackupsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.volumes = VolumesOperations(self._client, self._config, self._serialize, self._deserialize)
        self.snapshots = SnapshotsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.snapshot_policies = SnapshotPoliciesOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.backup_policies = BackupPoliciesOperations(self._client, self._config, self._serialize, self._deserialize)
        self.volume_quota_rules = VolumeQuotaRulesOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.ransomware_reports = RansomwareReportsOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.backup_vaults = BackupVaultsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.buckets = BucketsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.caches = CachesOperations(self._client, self._config, self._serialize, self._deserialize)
        self.elastic_accounts = ElasticAccountsOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.elastic_capacity_pools = ElasticCapacityPoolsOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.elastic_volumes = ElasticVolumesOperations(self._client, self._config, self._serialize, self._deserialize)
        self.elastic_snapshots = ElasticSnapshotsOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.elastic_snapshot_policies = ElasticSnapshotPoliciesOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.elastic_backup_vaults = ElasticBackupVaultsOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.elastic_backup_policies = ElasticBackupPoliciesOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.elastic_backups = ElasticBackupsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.active_directory_configs = ActiveDirectoryConfigsOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.net_app_resource_quota_limits = NetAppResourceQuotaLimitsOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.net_app_resource_region_infos = NetAppResourceRegionInfosOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.accounts = AccountsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.backups_under_account = BackupsUnderAccountOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.pools = PoolsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.backups_under_backup_vault = BackupsUnderBackupVaultOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.backups_under_volume = BackupsUnderVolumeOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.subvolumes = SubvolumesOperations(self._client, self._config, self._serialize, self._deserialize)
        self.net_app_resource = NetAppResourceOperations(self._client, self._config, self._serialize, self._deserialize)
        self.net_app_resource_usages = NetAppResourceUsagesOperations(
            self._client, self._config, self._serialize, self._deserialize
        )

    def send_request(self, request: HttpRequest, *, stream: bool = False, **kwargs: Any) -> HttpResponse:
        """Runs the network request through the client's chained policies.

        >>> from azure.core.rest import HttpRequest
        >>> request = HttpRequest("GET", "https://www.example.org/")
        <HttpRequest [GET], url: 'https://www.example.org/'>
        >>> response = client.send_request(request)
        <HttpResponse: 200 OK>

        For more information on this code flow, see https://aka.ms/azsdk/dpcodegen/python/send_request

        :param request: The network request you want to make. Required.
        :type request: ~azure.core.rest.HttpRequest
        :keyword bool stream: Whether the response payload will be streamed. Defaults to False.
        :return: The response of your network call. Does not do error handling on your response.
        :rtype: ~azure.core.rest.HttpResponse
        """

        request_copy = deepcopy(request)
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.base_url", self._config.base_url, "str", skip_quote=True),
        }

        request_copy.url = self._client.format_url(request_copy.url, **path_format_arguments)
        return self._client.send_request(request_copy, stream=stream, **kwargs)  # type: ignore

    def close(self) -> None:
        self._client.close()

    def __enter__(self) -> Self:
        self._client.__enter__()
        return self

    def __exit__(self, *exc_details: Any) -> None:
        self._client.__exit__(*exc_details)



# pylint: disable=too-many-lines
# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from enum import Enum
from azure.core import CaseInsensitiveEnumMeta


class AcceptGrowCapacityPoolForShortTermCloneSplit(  # pylint: disable=name-too-long
    str, Enum, metaclass=CaseInsensitiveEnumMeta
):
    """While auto splitting the short term clone volume, if the parent pool does not have enough space
    to accommodate the volume after split, it will be automatically resized, which will lead to
    increased billing. To accept capacity pool size auto grow and create a short term clone volume,
    set the property as accepted.
    """

    ACCEPTED = "Accepted"
    """Auto grow capacity pool for short term clone split is accepted."""
    DECLINED = "Declined"
    """Auto grow capacity pool for short term clone split is declined. Short term clone volume
    creation will not be allowed, to create short term clone volume accept auto grow capacity pool."""


class ActiveDirectoryStatus(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Status of the Active Directory."""

    CREATED = "Created"
    """Active Directory created but not in use."""
    IN_USE = "InUse"
    """Active Directory in use by SMB Volume."""
    DELETED = "Deleted"
    """Active Directory Deleted."""
    ERROR = "Error"
    """Error with the Active Directory."""
    UPDATING = "Updating"
    """Active Directory Updating."""


class ActualRansomwareProtectionState(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The actual state of the Advanced Ransomware Protection feature."""

    DISABLED = "Disabled"
    """Advanced Ransomware Protection is disabled."""
    ENABLED = "Enabled"
    """Advanced Ransomware Protection is enabled."""
    LEARNING = "Learning"
    """Advanced Ransomware Protection is in learning mode."""
    PAUSED = "Paused"
    """Advanced Ransomware Protection is in paused state."""


class ApplicationType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Application Type."""

    SAP_HANA = "SAP-HANA"
    """SAP_HANA."""
    ORACLE = "ORACLE"
    """ORACLE."""


class AvsDataStore(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Specifies whether the volume is enabled for Azure VMware Solution (AVS) datastore purpose."""

    ENABLED = "Enabled"
    """avsDataStore is enabled."""
    DISABLED = "Disabled"
    """avsDataStore is disabled."""


class BackupType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Type of backup Manual or Scheduled."""

    MANUAL = "Manual"
    """Manual backup."""
    SCHEDULED = "Scheduled"
    """Scheduled backup."""


class BindAuthenticationLevel(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The authentication level to use when binding to the LDAP server."""

    ANONYMOUS = "Anonymous"
    """Anonymous bind i.e. no credentials provided."""
    SIMPLE = "Simple"
    """Simple bind i.e. plain text credentials provided."""


class BreakthroughMode(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Specifies whether the volume operates in Breakthrough Mode. When set to 'Enabled', the volume
    runs on the resources configured for this mode, delivering improved performance and higher
    throughput. If set to 'Disabled' or omitted, the volume uses the basic configuration. This
    feature is available only in regions where it’s been configured and first-time users must
    finish onboarding prior to using Breakthrough Mode.
    """

    ENABLED = "Enabled"
    """The volume runs on the resources configured for Breakthrough mode which ensures consistent high
    performance and a higher throughput."""
    DISABLED = "Disabled"
    """The volume uses configuration that provides basic performance and throughput."""


class BucketPatchPermissions(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Access permissions for the bucket. Either ReadOnly or ReadWrite."""

    READ_ONLY = "ReadOnly"
    """Read-only access to bucket."""
    READ_WRITE = "ReadWrite"
    """Read-write access to bucket."""


class BucketPermissions(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Access permissions for the bucket. Either ReadOnly or ReadWrite. The default is ReadOnly if no
    value is provided during bucket creation.
    """

    READ_ONLY = "ReadOnly"
    """Read-only access to bucket."""
    READ_WRITE = "ReadWrite"
    """Read-write access to bucket."""


class CacheFileAccessLogs(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Flag indicating whether file access logs are enabled for the Cache, based on active diagnostic
    settings present on the Cache.
    """

    ENABLED = "Enabled"
    """fileAccessLogs are enabled."""
    DISABLED = "Disabled"
    """fileAccessLogs are not enabled."""


class CacheLifeCycleState(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Azure NetApp Files Cache lifecycle management."""

    CLUSTER_PEERING_OFFER_SENT = "ClusterPeeringOfferSent"
    """Cluster peering offer has been sent."""
    VSERVER_PEERING_OFFER_SENT = "VserverPeeringOfferSent"
    """VServer peering offer has been sent."""
    CREATING = "Creating"
    """Cache creation in progress."""
    SUCCEEDED = "Succeeded"
    """Cache creation succeeded and is available for use."""
    FAILED = "Failed"
    """Cache is in a failed state."""


class CacheProvisioningState(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Azure lifecycle management."""

    CREATING = "Creating"
    """The resource is being created."""
    UPDATING = "Updating"
    """The resource is being updated."""
    DELETING = "Deleting"
    """The resource is being deleted."""
    FAILED = "Failed"
    """The resource is in a failed state."""
    SUCCEEDED = "Succeeded"
    """The resource is succeeded."""
    CANCELED = "Canceled"
    """Resource creation was canceled."""


class CheckElasticResourceAvailabilityReason(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Availability reason."""

    INVALID = "Invalid"
    """Value indicating the name provided does not match Azure NetApp Files naming requirements."""
    ALREADY_EXISTS = "AlreadyExists"
    """Value indicating the name is already in use and is therefore unavailable."""


class CheckElasticResourceAvailabilityStatus(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Availability status."""

    TRUE = "True"
    """Value indicating the name is valid and available."""
    FALSE = "False"
    """Value indicating the the name is invalid, unavailable, or both."""


class CheckNameResourceTypes(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Resource type used for verification."""

    MICROSOFT_NET_APP_NET_APP_ACCOUNTS = "Microsoft.NetApp/netAppAccounts"
    """MICROSOFT_NET_APP_NET_APP_ACCOUNTS."""
    MICROSOFT_NET_APP_NET_APP_ACCOUNTS_CAPACITY_POOLS = "Microsoft.NetApp/netAppAccounts/capacityPools"
    """MICROSOFT_NET_APP_NET_APP_ACCOUNTS_CAPACITY_POOLS."""
    MICROSOFT_NET_APP_NET_APP_ACCOUNTS_CAPACITY_POOLS_VOLUMES = "Microsoft.NetApp/netAppAccounts/capacityPools/volumes"
    """MICROSOFT_NET_APP_NET_APP_ACCOUNTS_CAPACITY_POOLS_VOLUMES."""
    MICROSOFT_NET_APP_NET_APP_ACCOUNTS_CAPACITY_POOLS_VOLUMES_SNAPSHOTS = (
        "Microsoft.NetApp/netAppAccounts/capacityPools/volumes/snapshots"
    )
    """MICROSOFT_NET_APP_NET_APP_ACCOUNTS_CAPACITY_POOLS_VOLUMES_SNAPSHOTS."""
    MICROSOFT_NET_APP_NET_APP_ACCOUNTS_BACKUP_VAULTS_BACKUPS = "Microsoft.NetApp/netAppAccounts/backupVaults/backups"
    """ANF Backup under a volume , deprecated, use
    ``Microsoft.NetApp/netAppAccounts/backupVaults/backups`` instead."""
    MICROSOFT_NET_APP_NET_APP_ACCOUNTS_CAPACITY_POOLS_VOLUMES_BACKUPS = (
        "Microsoft.NetApp/netAppAccounts/capacityPools/volumes/backups"
    )
    """ANF Backup under a Backup Vault."""


class CheckQuotaNameResourceTypes(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Resource type used for verification."""

    MICROSOFT_NET_APP_NET_APP_ACCOUNTS = "Microsoft.NetApp/netAppAccounts"
    """MICROSOFT_NET_APP_NET_APP_ACCOUNTS."""
    MICROSOFT_NET_APP_NET_APP_ACCOUNTS_CAPACITY_POOLS = "Microsoft.NetApp/netAppAccounts/capacityPools"
    """MICROSOFT_NET_APP_NET_APP_ACCOUNTS_CAPACITY_POOLS."""
    MICROSOFT_NET_APP_NET_APP_ACCOUNTS_CAPACITY_POOLS_VOLUMES = "Microsoft.NetApp/netAppAccounts/capacityPools/volumes"
    """MICROSOFT_NET_APP_NET_APP_ACCOUNTS_CAPACITY_POOLS_VOLUMES."""
    MICROSOFT_NET_APP_NET_APP_ACCOUNTS_CAPACITY_POOLS_VOLUMES_SNAPSHOTS = (
        "Microsoft.NetApp/netAppAccounts/capacityPools/volumes/snapshots"
    )
    """MICROSOFT_NET_APP_NET_APP_ACCOUNTS_CAPACITY_POOLS_VOLUMES_SNAPSHOTS."""
    MICROSOFT_NET_APP_NET_APP_ACCOUNTS_BACKUP_VAULTS_BACKUPS = "Microsoft.NetApp/netAppAccounts/backupVaults/backups"
    """ANF Backup under a volume , deprecated, use
    ``Microsoft.NetApp/netAppAccounts/backupVaults/backups`` instead."""
    MICROSOFT_NET_APP_NET_APP_ACCOUNTS_CAPACITY_POOLS_VOLUMES_BACKUPS = (
        "Microsoft.NetApp/netAppAccounts/capacityPools/volumes/backups"
    )
    """ANF Backup under a Backup Vault."""


class ChownMode(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """This parameter specifies who is authorized to change the ownership of a file. restricted - Only
    root user can change the ownership of the file. unrestricted - Non-root users can change
    ownership of files that they own.
    """

    RESTRICTED = "Restricted"
    """RESTRICTED."""
    UNRESTRICTED = "Unrestricted"
    """UNRESTRICTED."""


class CifsChangeNotifyState(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Flag indicating whether a CIFS change notification is enabled for the cache."""

    DISABLED = "Disabled"
    """CIFS change notification is disabled."""
    ENABLED = "Enabled"
    """CIFS change notification is enabled."""


class CoolAccessRetrievalPolicy(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """coolAccessRetrievalPolicy determines the data retrieval behavior from the cool tier to standard
    storage based on the read pattern for cool access enabled volumes. The possible values for this
    field are: Default - Data will be pulled from cool tier to standard storage on random reads.
    This policy is the default. OnRead - All client-driven data read is pulled from cool tier to
    standard storage on both sequential and random reads. Never - No client-driven data is pulled
    from cool tier to standard storage.
    """

    DEFAULT = "Default"
    """DEFAULT."""
    ON_READ = "OnRead"
    """ON_READ."""
    NEVER = "Never"
    """NEVER."""


class CoolAccessTieringPolicy(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """coolAccessTieringPolicy determines which cold data blocks are moved to cool tier. The possible
    values for this field are: Auto - Moves cold user data blocks in both the Snapshot copies and
    the active file system to the cool tier tier. This policy is the default. SnapshotOnly - Moves
    user data blocks of the Volume Snapshot copies that are not associated with the active file
    system to the cool tier.
    """

    AUTO = "Auto"
    """AUTO."""
    SNAPSHOT_ONLY = "SnapshotOnly"
    """SNAPSHOT_ONLY."""


class CreatedByType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The kind of entity that created the resource."""

    USER = "User"
    """The entity was created by a user."""
    APPLICATION = "Application"
    """The entity was created by an application."""
    MANAGED_IDENTITY = "ManagedIdentity"
    """The entity was created by a managed identity."""
    KEY = "Key"
    """The entity was created by a key."""


class CredentialsStatus(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The bucket credentials status. There states:

    "NoCredentialsSet": Access and Secret key pair have not been generated.
    "CredentialsExpired": Access and Secret key pair have expired.
    "Active": The certificate has been installed and credentials are unexpired.
    """

    NO_CREDENTIALS_SET = "NoCredentialsSet"
    """Access and Secret key pair have not been generated."""
    CREDENTIALS_EXPIRED = "CredentialsExpired"
    """Access and Secret key pair have expired."""
    ACTIVE = "Active"
    """The certificate has been installed on the bucket server and the bucket credentials are
    unexpired."""


class DayOfWeek(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Day of the week."""

    SUNDAY = "Sunday"
    """Take a snapshot each Sunday."""
    MONDAY = "Monday"
    """Take a snapshot each Monday."""
    TUESDAY = "Tuesday"
    """Take a snapshot each Tuesday."""
    WEDNESDAY = "Wednesday"
    """Take a snapshot each Wednesday."""
    THURSDAY = "Thursday"
    """Take a snapshot each Thursday."""
    FRIDAY = "Friday"
    """Take a snapshot each Friday."""
    SATURDAY = "Saturday"
    """Take a snapshot each Saturday."""


class DesiredRansomwareProtectionState(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The desired state of the Advanced Ransomware Protection feature."""

    DISABLED = "Disabled"
    """Advanced Ransomware Protection is disabled."""
    ENABLED = "Enabled"
    """Advanced Ransomware Protection is enabled."""


class ElasticBackupPolicyState(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Elastic Backup Policy state."""

    ENABLED = "Enabled"
    """Value indicating the policy is enabled."""
    DISABLED = "Disabled"
    """Value indicating the policy is disabled."""


class ElasticBackupType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Type of backup."""

    MANUAL = "Manual"
    """Manual backup type."""
    SCHEDULED = "Scheduled"
    """Scheduled backup type."""


class ElasticKeyVaultStatus(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """KeyVault status."""

    CREATED = "Created"
    """KeyVault connection created but not in use."""
    IN_USE = "InUse"
    """KeyVault connection in use by SMB Volume."""
    DELETED = "Deleted"
    """KeyVault connection Deleted."""
    ERROR = "Error"
    """Error with the KeyVault connection."""
    UPDATING = "Updating"
    """KeyVault connection Updating."""


class ElasticNfsv3Access(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """NFSv3 access."""

    ENABLED = "Enabled"
    """Clients can connect to the volume using the NFSv3 protocol."""
    DISABLED = "Disabled"
    """Clients can't connect to the volume using the NFSv3 protocol."""


class ElasticNfsv4Access(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """NFSv4 access."""

    ENABLED = "Enabled"
    """Clients can connect to the volume using the NFSv4 protocol."""
    DISABLED = "Disabled"
    """Clients can't connect to the volume using the NFSv4 protocol."""


class ElasticPoolEncryptionKeySource(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Pool Encryption Key Source."""

    NET_APP = "NetApp"
    """Represents the encryption key source of Elastic pool is Microsoft.NetApp."""
    KEY_VAULT = "KeyVault"
    """Represents the encryption key source of Elastic pool is Microsoft.KeyVault."""


class ElasticProtocolType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Protocol types for elastic volume."""

    NF_SV3 = "NFSv3"
    """NFSv3 protocol type."""
    NF_SV4 = "NFSv4"
    """NFSv4 protocol type."""
    SMB = "SMB"
    """SMB/CIFS protocol type."""


class ElasticResourceAvailabilityStatus(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Current availability status of the resource."""

    ONLINE = "Online"
    """The resource is currently Online and accessible."""
    OFFLINE = "Offline"
    """The resource is currently Offline and not accessible."""


class ElasticRootAccess(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Root access."""

    ENABLED = "Enabled"
    """Root user access is enabled for clients affected by this rule."""
    DISABLED = "Disabled"
    """Root user access is disabled for clients affected by this rule."""


class ElasticServiceLevel(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Service level for elastic capacity pool."""

    ZONE_REDUNDANT = "ZoneRedundant"
    """Zone redundant storage service level."""


class ElasticSmbEncryption(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """SMB encryption."""

    ENABLED = "Enabled"
    """Value indicating the SMB encryption is enabled."""
    DISABLED = "Disabled"
    """Value indicating the SMB encryption is disabled."""


class ElasticUnixAccessRule(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Unix access rule."""

    READ_ONLY = "ReadOnly"
    """Clients connecting with this rule will only have read access to the volume."""
    READ_WRITE = "ReadWrite"
    """Clients connecting with this rule will have full read and write access to the volume."""
    NO_ACCESS = "NoAccess"
    """Clients connecting with this rule will have no access to the volume."""


class ElasticVolumePolicyEnforcement(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Policy enforcement."""

    ENFORCED = "Enforced"
    """Value indicating the policy is enforced on the volume."""
    NOT_ENFORCED = "NotEnforced"
    """Value indicating the policy is not enforced on the volume."""


class ElasticVolumeRestorationState(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The current state of the restoration process."""

    RESTORING = "Restoring"
    """Value indicating that the volume is currently restoring."""
    RESTORED = "Restored"
    """Value indicating that the volume is restored."""
    FAILED = "Failed"
    """Value indicating that the volume restore has failed."""


class EnableSubvolumes(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Flag indicating whether subvolume operations are enabled on the volume."""

    ENABLED = "Enabled"
    """subvolumes are enabled."""
    DISABLED = "Disabled"
    """subvolumes are not enabled."""


class EnableWriteBackState(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Flag indicating whether writeback is enabled for the cache."""

    DISABLED = "Disabled"
    """Writeback cache is disabled."""
    ENABLED = "Enabled"
    """Writeback cache is enabled."""


class EncryptionKeySource(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Source of key used to encrypt data in volume. Applicable if NetApp account has
    encryption.keySource = 'Microsoft.KeyVault'. Possible values (case-insensitive) are:
    'Microsoft.NetApp, Microsoft.KeyVault'.
    """

    MICROSOFT_NET_APP = "Microsoft.NetApp"
    """Microsoft-managed key encryption."""
    MICROSOFT_KEY_VAULT = "Microsoft.KeyVault"
    """Customer-managed key encryption."""


class EncryptionState(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Specifies if the cache is encryption or not."""

    DISABLED = "Disabled"
    """Encryption is disabled."""
    ENABLED = "Enabled"
    """Encryption is enabled."""


class EncryptionType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Encryption type of the capacity pool, set encryption type for data at rest for this pool and
    all volumes in it. This value can only be set when creating new pool.
    """

    SINGLE = "Single"
    """EncryptionType Single, volumes will use single encryption at rest."""
    DOUBLE = "Double"
    """EncryptionType Double, volumes will use double encryption at rest."""


class EndpointType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Indicates whether the local volume is the source or destination for the Volume Replication."""

    SRC = "src"
    """SRC."""
    DST = "dst"
    """DST."""


class Exclude(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """An option to filter out replications. 'None' returns all replications, 'Deleted' excludes
    deleted replications. Default is 'None'.
    """

    NONE = "None"
    """'None' returns all replications."""
    DELETED = "Deleted"
    """'Deleted' excludes deleted replications."""


class ExternalReplicationSetupStatus(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Property that only applies to external replications. Provides a machine-readable value for the
    status of the external replication setup.
    """

    CLUSTER_PEER_REQUIRED = "ClusterPeerRequired"
    """Your cluster needs to be peered by using the 'peerExternalCluster' action."""
    CLUSTER_PEER_PENDING = "ClusterPeerPending"
    """The peering needs to be accepted on your cluster before the setup can proceed."""
    V_SERVER_PEER_REQUIRED = "VServerPeerRequired"
    """Need to call 'authorizeExternalReplication' and accept the returned 'vserver peer accept'
    command on your cluster to finish setting up the external replication."""
    REPLICATION_CREATE_REQUIRED = "ReplicationCreateRequired"
    """Need to call 'authorizeExternalReplication' to finish setting up the external replication."""
    NO_ACTION_REQUIRED = "NoActionRequired"
    """External Replication setup is complete, you can now monitor the 'mirrorState' in the
    replication status for the health of the replication."""


class FileAccessLogs(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Flag indicating whether file access logs are enabled for the volume, based on active diagnostic
    settings present on the volume.
    """

    ENABLED = "Enabled"
    """fileAccessLogs are enabled."""
    DISABLED = "Disabled"
    """fileAccessLogs are not enabled."""


class GlobalFileLockingState(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Flag indicating whether the global file lock is enabled for the cache."""

    DISABLED = "Disabled"
    """Global file locking is disabled."""
    ENABLED = "Enabled"
    """Global file locking is enabled."""


class InAvailabilityReasonType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """<code>Invalid</code> indicates the name provided does not match Azure App Service naming
    requirements. <code>AlreadyExists</code> indicates that the name is already in use and is
    therefore unavailable.
    """

    INVALID = "Invalid"
    """INVALID."""
    ALREADY_EXISTS = "AlreadyExists"
    """ALREADY_EXISTS."""


class KerberosState(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Describe if a cache is Kerberos enabled."""

    DISABLED = "Disabled"
    """Kerberos is disabled."""
    ENABLED = "Enabled"
    """Kerberos is enabled."""


class KeySource(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The encryption keySource (provider). Possible values (case-insensitive):  Microsoft.NetApp,
    Microsoft.KeyVault.
    """

    MICROSOFT_NET_APP = "Microsoft.NetApp"
    """Microsoft-managed key encryption."""
    MICROSOFT_KEY_VAULT = "Microsoft.KeyVault"
    """Customer-managed key encryption."""


class KeyVaultStatus(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Status of the KeyVault connection."""

    CREATED = "Created"
    """KeyVault connection created but not in use."""
    IN_USE = "InUse"
    """KeyVault connection in use by SMB Volume."""
    DELETED = "Deleted"
    """KeyVault connection Deleted."""
    ERROR = "Error"
    """Error with the KeyVault connection."""
    UPDATING = "Updating"
    """KeyVault connection Updating."""


class LargeVolumeType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Specifies the type of the Large Volume. When set to 'LargeVolume', the large volume is created
    with standard configuration. If it is set to 'ExtraLargeVolume7Dot2PiB', the extra large volume
    is created with higher capacity limit 7.2PiB with cool access enabled, delivering higher
    capacity limit with lower costs.
    """

    LARGE_VOLUME = "LargeVolume"
    """The large volume is created with standard configuration that provides standard performance and
    throughput."""
    EXTRA_LARGE_VOLUME7_DOT2_PI_B = "PremExtraLargeVolume7Dot2PiB"
    """The extra large volume is created with higher volume capacity limit 7.2PiB with cool access
    enabled, delivering higher capacity limit with lower costs."""


class LdapServerType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The type of the LDAP server."""

    ACTIVE_DIRECTORY = "ActiveDirectory"
    """The volume should use Active Directory for LDAP connections."""
    OPEN_LDAP = "OpenLDAP"
    """The volume should use OpenLDAP for LDAP connections."""


class LdapState(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Specifies whether LDAP is enabled or not."""

    DISABLED = "Disabled"
    """ldap is disabled."""
    ENABLED = "Enabled"
    """ldap is enabled."""


class ManagedServiceIdentityType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Type of managed service identity (where both SystemAssigned and UserAssigned types are
    allowed).
    """

    NONE = "None"
    """No managed identity."""
    SYSTEM_ASSIGNED = "SystemAssigned"
    """System assigned managed identity."""
    USER_ASSIGNED = "UserAssigned"
    """User assigned managed identity."""
    SYSTEM_ASSIGNED_USER_ASSIGNED = "SystemAssigned,UserAssigned"
    """System and user assigned managed identity."""


class MetricAggregationType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Type of MetricAggregationType."""

    AVERAGE = "Average"
    """AVERAGE."""


class MirrorState(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The status of the replication."""

    UNINITIALIZED = "Uninitialized"
    """UNINITIALIZED."""
    MIRRORED = "Mirrored"
    """MIRRORED."""
    BROKEN = "Broken"
    """BROKEN."""


class MultiAdStatus(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """MultiAD Status for the account."""

    DISABLED = "Disabled"
    """Account is MultiAD disabled, Means its a SharedAD or SingleAD account."""
    ENABLED = "Enabled"
    """Account is MultiAD enabled."""


class NetAppProvisioningState(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Gets the status of the VolumeQuotaRule at the time the operation was called."""

    ACCEPTED = "Accepted"
    """Resource has been Accepted."""
    CREATING = "Creating"
    """Resource is being Created."""
    PATCHING = "Patching"
    """Resource is being Patched."""
    UPDATING = "Updating"
    """Resource is updating."""
    DELETING = "Deleting"
    """Resource is being Deleted."""
    MOVING = "Moving"
    """Resource is being Moved."""
    FAILED = "Failed"
    """Resource has Failed."""
    SUCCEEDED = "Succeeded"
    """Resource has Succeeded."""


class NetworkFeatures(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Network features available to the volume, or current state of update."""

    BASIC = "Basic"
    """Basic network features."""
    STANDARD = "Standard"
    """Standard network features."""
    BASIC_STANDARD = "Basic_Standard"
    """Updating from Basic to Standard network features."""
    STANDARD_BASIC = "Standard_Basic"
    """Updating from Standard to Basic network features."""


class NetworkSiblingSetProvisioningState(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Gets the status of the NetworkSiblingSet at the time the operation was called."""

    SUCCEEDED = "Succeeded"
    """SUCCEEDED."""
    FAILED = "Failed"
    """FAILED."""
    CANCELED = "Canceled"
    """CANCELED."""
    UPDATING = "Updating"
    """UPDATING."""


class OnCertificateConflictAction(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """This action is triggered when a certificate conflict occurs. A conflict arises if you try to
    create a new bucket while one or more already exist on the server, or if you update a bucket
    when multiple buckets are present. This happens because a single certificate is shared among
    all buckets on the same server.

    Note: This applies both to certificates provided directly via the certificateObject property
    and to those retrieved from Azure Key Vault. Details for the latter case are specified in the
    akvDetails.certificateAkvDetails section.
    """

    UPDATE = "Update"
    """Update the existing certificate regardless of whether there is a conflict or not. This means
    all buckets on the server will now use the new certificate."""
    FAIL = "Fail"
    """Fail the operation if a conflict occurs, meaning the bucket operation will fail, and the
    existing certificate will continue to be in use."""


class PolicyStatus(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Policy status."""

    ENABLED = "Enabled"
    """Value indicating the policy is enabled."""
    DISABLED = "Disabled"
    """Value indicating the policy is disabled."""


class ProtocolTypes(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Set of supported protocol types, which include NFSv3, NFSv4 and SMB protocol."""

    NF_SV3 = "NFSv3"
    """NFSv3 protocol type."""
    NF_SV4 = "NFSv4"
    """NFSv4 protocol type."""
    SMB = "SMB"
    """SMB protocol type."""


class QosType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The qos type of the pool."""

    AUTO = "Auto"
    """qos type Auto."""
    MANUAL = "Manual"
    """qos type Manual."""


class QuotaType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Type of quota."""

    DEFAULT_USER_QUOTA = "DefaultUserQuota"
    """Default user quota."""
    DEFAULT_GROUP_QUOTA = "DefaultGroupQuota"
    """Default group quota."""
    INDIVIDUAL_USER_QUOTA = "IndividualUserQuota"
    """Individual user quota."""
    INDIVIDUAL_GROUP_QUOTA = "IndividualGroupQuota"
    """Individual group quota."""


class RansomwareReportSeverity(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Severity of the Advanced Ransomware Protection (ARP) report."""

    NONE = "None"
    """No data is suspected for ransomware activity."""
    LOW = "Low"
    """Low attack probability."""
    MODERATE = "Moderate"
    """Moderate attack probability."""
    HIGH = "High"
    """High amount of data is suspected for ransomware activity."""


class RansomwareReportState(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """State of the Advanced Ransomware Protection (ARP) report."""

    ACTIVE = "Active"
    """The ARP report has been created. Take action by running clearsuspects marking suspects as
    FalsePositive or PotentialThreats."""
    RESOLVED = "Resolved"
    """The ARP Report has been resolved."""


class RansomwareSuspectResolution(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """ARP report suspect resolution."""

    POTENTIAL_THREAT = "PotentialThreat"
    """The identified file type is unexpected in your workload and should be treated as a potential
    attack."""
    FALSE_POSITIVE = "FalsePositive"
    """The identified file type is expected in your workload and can be ignored."""


class RegionStorageToNetworkProximity(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Provides storage to network proximity information in the region."""

    DEFAULT = "Default"
    """Basic network connectivity."""
    T1 = "T1"
    """Standard T1 network connectivity."""
    T2 = "T2"
    """Standard T2 network connectivity."""
    ACROSS_T2 = "AcrossT2"
    """Standard AcrossT2 network connectivity."""
    T1_AND_T2 = "T1AndT2"
    """Standard T1 and T2 network connectivity."""
    T1_AND_ACROSS_T2 = "T1AndAcrossT2"
    """Standard T1 and AcrossT2 network connectivity."""
    T2_AND_ACROSS_T2 = "T2AndAcrossT2"
    """Standard T2 and AcrossT2 network connectivity."""
    T1_AND_T2_AND_ACROSS_T2 = "T1AndT2AndAcrossT2"
    """Standard T1, T2 and AcrossT2 network connectivity."""


class ReplicationMirrorState(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The status of the replication."""

    UNINITIALIZED = "Uninitialized"
    """Destination volume has not been initialized."""
    MIRRORED = "Mirrored"
    """Destination volume has been initialized and is ready."""
    BROKEN = "Broken"
    """Destination volume is RW, replication relationship has been broken off."""


class ReplicationSchedule(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Schedule."""

    ENUM_10_MINUTELY = "_10minutely"
    """10_MINUTELY."""
    HOURLY = "hourly"
    """HOURLY."""
    DAILY = "daily"
    """DAILY."""


class ReplicationType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Indicates whether the replication is cross zone or cross region."""

    CROSS_REGION_REPLICATION = "CrossRegionReplication"
    """Cross region replication."""
    CROSS_ZONE_REPLICATION = "CrossZoneReplication"
    """Cross zone replication."""


class SecurityStyle(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The security style of volume, default unix, defaults to ntfs for dual protocol or CIFS
    protocol.
    """

    NTFS = "ntfs"
    """NTFS."""
    UNIX = "unix"
    """UNIX."""


class ServiceLevel(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The service level of the file system."""

    STANDARD = "Standard"
    """Standard service level."""
    PREMIUM = "Premium"
    """Premium service level."""
    ULTRA = "Ultra"
    """Ultra service level."""
    STANDARD_ZRS = "StandardZRS"
    """Zone redundant storage service level. This will be deprecated soon."""
    FLEXIBLE = "Flexible"
    """Flexible service level."""


class SmbAccessBasedEnumeration(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Enables access-based enumeration share property for SMB Shares. Only applicable for
    SMB/DualProtocol volume.
    """

    DISABLED = "Disabled"
    """smbAccessBasedEnumeration share setting is disabled."""
    ENABLED = "Enabled"
    """smbAccessBasedEnumeration share setting is enabled."""


class SmbEncryptionState(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Enables encryption for in-flight smb3 data. Only applicable for SMB/DualProtocol cache."""

    DISABLED = "Disabled"
    """SMB encryption is disabled."""
    ENABLED = "Enabled"
    """SMB encryption is enabled."""


class SmbNonBrowsable(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Enables non-browsable property for SMB Shares. Only applicable for SMB/DualProtocol volume."""

    DISABLED = "Disabled"
    """smbNonBrowsable share setting is disabled."""
    ENABLED = "Enabled"
    """smbNonBrowsable share setting is enabled."""


class SnapshotDirectoryVisibility(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Controls the visibility of the Elastic Volume's read-only snapshot directory, which provides
    access to each of the volume's snapshots.
    """

    HIDDEN = "Hidden"
    """Value indicating the read-only snapshot directory is not visible."""
    VISIBLE = "Visible"
    """Value indicating the read-only snapshot directory is visible."""


class SnapshotUsage(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Snapshot usage for backup."""

    USE_EXISTING_SNAPSHOT = "UseExistingSnapshot"
    """Value indicating an existing snapshot is used."""
    CREATE_NEW_SNAPSHOT = "CreateNewSnapshot"
    """Value indicating a new snapshot is created."""


class VolumeBackupRelationshipStatus(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Status of the volume backup relationship."""

    IDLE = "Idle"
    """IDLE."""
    TRANSFERRING = "Transferring"
    """TRANSFERRING."""
    FAILED = "Failed"
    """FAILED."""
    UNKNOWN = "Unknown"
    """UNKNOWN."""


class VolumeLanguage(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Language supported for volume."""

    C_UTF8 = "c.utf-8"
    """Posix with UTF-8."""
    UTF8_MB4 = "utf8mb4"
    """UTF-8 with 4 byte character support."""
    AR = "ar"
    """Arabic - Deprecated."""
    AR_UTF8 = "ar.utf-8"
    """Arabic with UTF-8."""
    HR = "hr"
    """Croatian - Deprecated."""
    HR_UTF8 = "hr.utf-8"
    """Croatian with UTF-8."""
    CS = "cs"
    """Czech - Deprecated."""
    CS_UTF8 = "cs.utf-8"
    """Czech with UTF-8."""
    DA = "da"
    """Danish - Deprecated."""
    DA_UTF8 = "da.utf-8"
    """Danish with UTF-8."""
    NL = "nl"
    """Dutch - Deprecated."""
    NL_UTF8 = "nl.utf-8"
    """Dutch with UTF-8."""
    EN = "en"
    """English - Deprecated."""
    EN_UTF8 = "en.utf-8"
    """English with UTF-8."""
    FI = "fi"
    """Finnish - Deprecated."""
    FI_UTF8 = "fi.utf-8"
    """Finnish with UTF-8."""
    FR = "fr"
    """French - Deprecated."""
    FR_UTF8 = "fr.utf-8"
    """French with UTF-8."""
    DE = "de"
    """German - Deprecated."""
    DE_UTF8 = "de.utf-8"
    """German with UTF-8."""
    HE = "he"
    """Hebrew - Deprecated."""
    HE_UTF8 = "he.utf-8"
    """Hebrew with UTF-8."""
    HU = "hu"
    """Hungarian - Deprecated."""
    HU_UTF8 = "hu.utf-8"
    """Hungarian with UTF-8."""
    IT = "it"
    """Italian - Deprecated."""
    IT_UTF8 = "it.utf-8"
    """Italian with UTF-8."""
    JA = "ja"
    """Japanese euc-j - Deprecated."""
    JA_UTF8 = "ja.utf-8"
    """Japanese euc-j with UTF-8."""
    JA_V1 = "ja-v1"
    """Japanese euc-j - Deprecated."""
    JA_V1_UTF8 = "ja-v1.utf-8"
    """Japanese euc-j with UTF-8."""
    JA_JP_PCK = "ja-jp.pck"
    """Japanese pck."""
    JA_JP_PCK_UTF8 = "ja-jp.pck.utf-8"
    """Japanese pck with UTF-8 - Deprecated."""
    JA_JP932 = "ja-jp.932"
    """Japanese cp932."""
    JA_JP932_UTF8 = "ja-jp.932.utf-8"
    """Japanese cp932 with UTF-8 - Deprecated."""
    JA_JP_PCK_V2 = "ja-jp.pck-v2"
    """Japanese pck - sjis."""
    JA_JP_PCK_V2_UTF8 = "ja-jp.pck-v2.utf-8"
    """Japanese pck - sjis with UTF-8 - Deprecated."""
    KO = "ko"
    """Korean - Deprecated."""
    KO_UTF8 = "ko.utf-8"
    """Korean with UTF-8."""
    NO = "no"
    """Norwegian - Deprecated."""
    NO_UTF8 = "no.utf-8"
    """Norwegian with UTF-8."""
    PL = "pl"
    """Polish - Deprecated."""
    PL_UTF8 = "pl.utf-8"
    """Polish with UTF-8."""
    PT = "pt"
    """Portuguese - Deprecated."""
    PT_UTF8 = "pt.utf-8"
    """Portuguese with UTF-8."""
    C = "c"
    """Posix - Deprecated."""
    RO = "ro"
    """Romanian - Deprecated."""
    RO_UTF8 = "ro.utf-8"
    """Romanian with UTF-8."""
    RU = "ru"
    """Russian - Deprecated."""
    RU_UTF8 = "ru.utf-8"
    """Russian with UTF-8."""
    ZH = "zh"
    """Simplified Chinese - Deprecated."""
    ZH_UTF8 = "zh.utf-8"
    """Simplified Chinese with UTF-8."""
    ZH_GBK = "zh.gbk"
    """Simplified gbk Chinese."""
    ZH_GBK_UTF8 = "zh.gbk.utf-8"
    """Simplified gbk Chinese with UTF-8 - Deprecated."""
    ZH_TW_BIG5 = "zh-tw.big5"
    """Traditional Chinese BIG 5."""
    ZH_TW_BIG5_UTF8 = "zh-tw.big5.utf-8"
    """Traditional Chinese BIG 5 with UTF-8 - Deprecated."""
    ZH_TW = "zh-tw"
    """Traditional Chinese EUC-TW."""
    ZH_TW_UTF8 = "zh-tw.utf-8"
    """Traditional Chinese EUC-TW with UTF-8 - Deprecated."""
    SK = "sk"
    """Slovak - Deprecated."""
    SK_UTF8 = "sk.utf-8"
    """Slovak with UTF-8."""
    SL = "sl"
    """Slovenian - Deprecated."""
    SL_UTF8 = "sl.utf-8"
    """Slovenian with UTF-8."""
    ES = "es"
    """Spanish - Deprecated."""
    ES_UTF8 = "es.utf-8"
    """Spanish with UTF-8."""
    SV = "sv"
    """Swedish - Deprecated."""
    SV_UTF8 = "sv.utf-8"
    """Swedish with UTF-8."""
    TR = "tr"
    """Turkish - Deprecated."""
    TR_UTF8 = "tr.utf-8"
    """Turkish with UTF-8."""
    EN_US = "en-us"
    """US English - Deprecated."""
    EN_US_UTF8 = "en-us.utf-8"
    """US English with UTF-8."""


class VolumeReplicationRelationshipStatus(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Status of the volume replication relationship."""

    IDLE = "Idle"
    """IDLE."""
    TRANSFERRING = "Transferring"
    """TRANSFERRING."""


class VolumeRestoreRelationshipStatus(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Status of the volume restore relationship."""

    IDLE = "Idle"
    """IDLE."""
    TRANSFERRING = "Transferring"
    """TRANSFERRING."""
    FAILED = "Failed"
    """FAILED."""
    UNKNOWN = "Unknown"
    """UNKNOWN."""


class VolumeSize(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Volume size for backup."""

    LARGE = "Large"
    """Value indicating backup is for a large volume."""
    REGULAR = "Regular"
    """Value indicating backup is not for a large volume."""


class VolumeStorageToNetworkProximity(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Provides storage to network proximity information for the volume."""

    DEFAULT = "Default"
    """Basic storage to network connectivity."""
    T1 = "T1"
    """Standard T1 storage to network connectivity."""
    T2 = "T2"
    """Standard T2 storage to network connectivity."""
    ACROSS_T2 = "AcrossT2"
    """Standard AcrossT2 storage to network connectivity."""



# resource constants
# some minor setup required to run in record mode
#  - actual subsid to be used
#  - rg must exist
#  - vnet must exist
LOCATION = "westus2"
REMOTE_LOCATION = "eastus2"
TEST_RG = "sdk-python-tests-rg-tmp"
TEST_REPL_REMOTE_RG = "sdk-python-tests-rg-R-tmp"
TEST_ACC_1 = "sdk-py-tests-acc-1"
TEST_ACC_2 = "sdk-py-tests-acc-2"
TEST_POOL_1 = "sdk-py-tests-pool-1"
TEST_POOL_2 = "sdk-py-tests-pool-2"
TEST_VOL_1 = "sdk-py-tests-vol-3"
TEST_VOL_2 = "sdk-py-tests-vol-4"
TEST_BACKUP_1 = "sdk-py-tests-backups-1"
TEST_BACKUP_2 = "sdk-py-tests-backups-2"
TEST_SUBVOLUME_1 = "sdk-py-tests-subvolume-1"
TEST_SUBVOLUME_2 = "sdk-py-tests-subvolume-2"
TEST_SNAPSHOT_1 = "sdk-py-tests-snapshot-1"
TEST_SNAPSHOT_2 = "sdk-py-tests-snapshot-2"
SERVICE_LEVEL = "Premium"
REMOTE_VNET = "sdk-python-tests-VNET-R"
PERMA_ACCOUNT = "sdk-python-perma-account"
PERMA_REMOTE_ACCOUNT = "sdk-python-perma-remote-account"
PERMA_POOL = "sdk-python-perma-pool"
PERMA_REMOTE_POOL = "sdk-python-perma-remote-pool"
PERMA_VOLUME = "sdk-python-perma-volume"
PERMA_REMOTE_VOLUME = "sdk-python-perma-remote-volume"
PERMA_VNET = "sdk-python-tests-perma-VNET"
PERMA_REMOTE_VNET = "sdk-python-tests-remote-perma-VNET"
PERMA_SUBNET = "sdk-python-tests-subnet"
PERMA_REMOTE_SUBNET = "sdk-python-tests-remote-subnet"
LIVE = False
SUBSID = "69a75bda-882e-44d5-8431-63421204132a"



import random
import string
from azure.mgmt.netapp.models import SubvolumeInfo, SubvolumePatchRequest, SubvolumePatchParams
from azure.core.exceptions import HttpResponseError
from devtools_testutils import AzureMgmtRecordedTestCase, recorded_by_proxy, set_bodiless_matcher
from test_volume import (
    create_volume,
    delete_volume,
    delete_pool,
    delete_account,
    create_virtual_network,
    wait_for_volume,
)
import setup
import azure.mgmt.netapp.models
import time


def create_subvolume(client, rg, account_name, pool_name, volume_name, subvolume_name, path, size):
    print("Creating subvolume {0} under volume {1}".format(subvolume_name, volume_name))
    subvolume_properties = azure.mgmt.netapp.models.SubvolumeProperties(path=path, size=size)
    subvolume_body = SubvolumeInfo(properties=subvolume_properties)

    client.subvolumes.begin_create(rg, account_name, pool_name, volume_name, subvolume_name, subvolume_body).result()
    subvolume = wait_for_subvolume(client, rg, account_name, pool_name, volume_name, subvolume_name)
    print("\tdone")
    return subvolume


def wait_for_subvolume(client, rg, account_name, pool_name, volume_name, subvolume_name):
    retry = 0
    while retry < 60:
        retry += 1
        subvolume = client.subvolumes.get(rg, account_name, pool_name, volume_name, subvolume_name)
        if subvolume.provisioning_state == "Succeeded":
            break
        if subvolume.provisioning_state == "Failed":
            print("\t Wait for subvolume. Subvolume is in a failed state.")
            break
        if setup.LIVE:
            time.sleep(3)
    if retry == 60:
        raise Exception("Timeout when waiting for subvolume")
    return subvolume


def delete_subvolume(client, rg, account_name, pool_name, volume_name, subvolume_name):
    print("Delete subvolume {0}".format(volume_name))
    client.subvolumes.begin_delete(rg, account_name, pool_name, volume_name, subvolume_name).wait()
    wait_for_no_subvolume(client, rg, account_name, pool_name, volume_name, subvolume_name)
    print("\tDone")


def wait_for_no_subvolume(client, rg, account_name, pool_name, volume_name, subvolume_name):
    retry = 0
    while retry < 60:
        retry += 1
        try:
            vol = client.subvolumes.get(rg, account_name, pool_name, volume_name, subvolume_name)
            print("\tGot subvolume. Still waiting. State = " + vol.provisioning_state)
        except:
            # not found is an exception case (status code 200 expected)
            # but is what we are waiting for
            break
        if setup.LIVE:
            time.sleep(3)
    if retry == 60:
        raise Exception("Timeout when waiting for no subvolume")


class TestNetAppSubvolume(AzureMgmtRecordedTestCase):

    def setup_method(self, method):
        self.client = self.create_mgmt_client(azure.mgmt.netapp.NetAppManagementClient)
        if self.is_live:
            setup.LIVE = True

    # Before tests are run live a resource group needs to be created along with vnet and subnet
    # Note that when tests are run in live mode it is best to run one test at a time.
    @recorded_by_proxy
    def test_crud_subvolumes(self):
        print("Starting test_crud_subvolumes")
        set_bodiless_matcher()
        volumeName1 = self.get_resource_name(setup.TEST_VOL_1 + "-")
        subvolumeName = self.get_resource_name(setup.TEST_SUBVOLUME_1 + "-")

        try:
            create_volume(
                self.client,
                setup.TEST_RG,
                setup.PERMA_ACCOUNT,
                setup.PERMA_POOL,
                volumeName1,
                setup.LOCATION,
                vnet=setup.PERMA_VNET,
                volume_only=True,
                enable_subvolumes="Enabled",
            )

            # create
            subvolume = create_subvolume(
                self.client,
                setup.TEST_RG,
                setup.PERMA_ACCOUNT,
                setup.PERMA_POOL,
                volumeName1,
                subvolumeName,
                "/sub_vol_1.txt",
                1000000,
            )

            assert (
                subvolume.name == setup.PERMA_ACCOUNT + "/" + setup.PERMA_POOL + "/" + volumeName1 + "/" + subvolumeName
            )
            assert subvolume.path == "/sub_vol_1.txt"

            # update
            ## Fix sdk gen, request is malformed for now
            subvolume_patch_properties = SubvolumePatchParams(path="/sub_vol_update.txt", size=2000000)
            subvolume_patch = SubvolumePatchRequest(properties=subvolume_patch_properties)
            print("Updating subvolume")
            update_succeeded = False
            try:
                subvolume = self.client.subvolumes.begin_update(
                    setup.TEST_RG, setup.PERMA_ACCOUNT, setup.PERMA_POOL, volumeName1, subvolumeName, subvolume_patch
                ).result()
                update_succeeded = True
                print("\tDone")
                wait_for_subvolume(
                    self.client, setup.TEST_RG, setup.PERMA_ACCOUNT, setup.PERMA_POOL, volumeName1, subvolumeName
                )
                wait_for_volume(self.client, setup.TEST_RG, setup.PERMA_ACCOUNT, setup.PERMA_POOL, volumeName1)
                assert (
                    subvolume.name
                    == setup.PERMA_ACCOUNT + "/" + setup.PERMA_POOL + "/" + volumeName1 + "/" + subvolumeName
                )
                assert subvolume.path == "/sub_vol_update.txt"
            except HttpResponseError as ex:
                # Known issue: generated request can be rejected as multipart/form-data.
                assert "multipart/form-data" in str(ex)

            # get
            if update_succeeded:
                subvolume_info = self.client.subvolumes.get(
                    setup.TEST_RG, setup.PERMA_ACCOUNT, setup.PERMA_POOL, volumeName1, subvolumeName
                )
                assert (
                    subvolume_info.name
                    == setup.PERMA_ACCOUNT + "/" + setup.PERMA_POOL + "/" + volumeName1 + "/" + subvolumeName
                )
                # assert subvolume_info.path == "/sub_vol_update.txt"
        finally:
            # delete
            delete_subvolume(
                self.client, setup.TEST_RG, setup.PERMA_ACCOUNT, setup.PERMA_POOL, volumeName1, subvolumeName
            )
            subvolume_list = self.client.subvolumes.list_by_volume(
                setup.TEST_RG, setup.PERMA_ACCOUNT, setup.PERMA_POOL, volumeName1
            )
            assert len(list(subvolume_list)) == 0
            delete_volume(self.client, setup.TEST_RG, setup.PERMA_ACCOUNT, setup.PERMA_POOL, volumeName1)

        print("Finished with test_crud_subvolumes")

    @recorded_by_proxy
    def test_list_by_volume(self):
        print("Starting test_list_by_volume")
        set_bodiless_matcher()
        volumeName1 = self.get_resource_name(setup.TEST_VOL_1 + "-")
        subvolumeName = self.get_resource_name(setup.TEST_SUBVOLUME_1 + "-")
        subvolumeName2 = self.get_resource_name(setup.TEST_SUBVOLUME_2 + "-")

        try:
            create_volume(
                self.client,
                setup.TEST_RG,
                setup.PERMA_ACCOUNT,
                setup.PERMA_POOL,
                volumeName1,
                setup.LOCATION,
                vnet=setup.PERMA_VNET,
                enable_subvolumes="Enabled",
            )

            # create
            subvolume = create_subvolume(
                self.client,
                setup.TEST_RG,
                setup.PERMA_ACCOUNT,
                setup.PERMA_POOL,
                volumeName1,
                subvolumeName,
                "/sub_vol_2.txt",
                1000000,
            )
            subvolume2 = create_subvolume(
                self.client,
                setup.TEST_RG,
                setup.PERMA_ACCOUNT,
                setup.PERMA_POOL,
                volumeName1,
                subvolumeName2,
                "/sub_vol_3.txt",
                2000000,
            )

            # list_by_volume
            subvolume_list = self.client.subvolumes.list_by_volume(
                setup.TEST_RG, setup.PERMA_ACCOUNT, setup.PERMA_POOL, volumeName1
            )
            assert len(list(subvolume_list)) == 2

            delete_subvolume(
                self.client, setup.TEST_RG, setup.PERMA_ACCOUNT, setup.PERMA_POOL, volumeName1, subvolumeName
            )
            subvolume_list = self.client.subvolumes.list_by_volume(
                setup.TEST_RG, setup.PERMA_ACCOUNT, setup.PERMA_POOL, volumeName1
            )
            assert len(list(subvolume_list)) == 1

            delete_subvolume(
                self.client, setup.TEST_RG, setup.PERMA_ACCOUNT, setup.PERMA_POOL, volumeName1, subvolumeName2
            )
            subvolume_list = self.client.subvolumes.list_by_volume(
                setup.TEST_RG, setup.PERMA_ACCOUNT, setup.PERMA_POOL, volumeName1
            )
            assert len(list(subvolume_list)) == 0
        finally:
            # clean up
            delete_volume(self.client, setup.TEST_RG, setup.PERMA_ACCOUNT, setup.PERMA_POOL, volumeName1)

        print("Finished with test_list_by_volume")

    @recorded_by_proxy
    def test_get_metadata(self):
        print("Starting test_get_metadata")
        set_bodiless_matcher()
        volumeName1 = self.get_resource_name(setup.TEST_VOL_1 + "-")
        subvolumeName = self.get_resource_name(setup.TEST_SUBVOLUME_1 + "-")

        try:
            create_volume(
                self.client,
                setup.TEST_RG,
                setup.PERMA_ACCOUNT,
                setup.PERMA_POOL,
                volumeName1,
                setup.LOCATION,
                vnet=setup.PERMA_VNET,
                enable_subvolumes="Enabled",
            )

            # create
            subvolume = create_subvolume(
                self.client,
                setup.TEST_RG,
                setup.PERMA_ACCOUNT,
                setup.PERMA_POOL,
                volumeName1,
                subvolumeName,
                "/sub_vol_4.txt",
                123,
            )
            assert (
                subvolume.name == setup.PERMA_ACCOUNT + "/" + setup.PERMA_POOL + "/" + volumeName1 + "/" + subvolumeName
            )
            assert subvolume.path == "/sub_vol_4.txt"

            # get metadata
            metadata = self.client.subvolumes.begin_get_metadata(
                setup.TEST_RG, setup.PERMA_ACCOUNT, setup.PERMA_POOL, volumeName1, subvolumeName
            ).result()
            assert metadata is not None

        finally:
            # clean up
            delete_subvolume(
                self.client, setup.TEST_RG, setup.PERMA_ACCOUNT, setup.PERMA_POOL, volumeName1, subvolumeName
            )
            delete_volume(self.client, setup.TEST_RG, setup.PERMA_ACCOUNT, setup.PERMA_POOL, volumeName1)

        print("Finished with test_get_metadata")



import time
from azure.mgmt.resource.resources import ResourceManagementClient
from devtools_testutils import AzureMgmtRecordedTestCase, recorded_by_proxy
import azure.mgmt.netapp.models
from azure.mgmt.netapp.models import NetAppAccount, AccountProperties, NetAppAccountPatch
import setup


def create_account(client, rg, acc_name, location=setup.LOCATION, tags=None, active_directories=None):
    account_body_properties = AccountProperties(active_directories=active_directories)
    account_body = NetAppAccount(location=location, tags=tags, properties=account_body_properties)

    print("Creating NetApp Account {0}".format(acc_name))
    account = client.accounts.begin_create_or_update(rg, acc_name, account_body).result()
    wait_for_account(client, rg, acc_name)
    print("\tDone creating NetApp Account {0}".format(acc_name))
    return account


def wait_for_account(client, rg, acc_name):
    # a work around for the async nature of certain ARM processes
    retry = 0
    while retry < 60:
        account = client.accounts.get(rg, acc_name)
        if account.provisioning_state == "Succeeded":
            break
        retry += 1
        if setup.LIVE:
            time.sleep(3)
    if retry == 60:
        raise Exception("Timeout when waiting for account")


def wait_for_no_account(client, rg, acc_name):
    # a workaround for the async nature of certain ARM processes
    retry = 0
    while retry < 60:
        try:
            client.accounts.get(rg, acc_name)
        except:
            # not found is an exception case (status code 200 expected)
            # and is actually what we are waiting for
            break
        retry += 1
        if setup.LIVE:
            time.sleep(3)
    if retry == 60:
        raise Exception("Timeout when waiting for no account")


def delete_account(client, rg, acc_name):
    print("Deleting NetApp Account {0}".format(acc_name))
    retry = 0
    while retry < 5:
        try:
            client.accounts.begin_delete(rg, acc_name).wait()
            break
        except Exception as e:
            print("failed to delete account. Retry number: {0}".format(retry))
            print(e)
            retry += 1
            if setup.LIVE:
                time.sleep(10)
    if retry == 5:
        raise Exception("Timeout when deleting account")

    wait_for_no_account(client, rg, acc_name)
    print("\tDone deleting account {0}".format(acc_name))


class TestNetAppAccount(AzureMgmtRecordedTestCase):

    def setup_method(self, method):
        self.client = self.create_mgmt_client(azure.mgmt.netapp.NetAppManagementClient)
        print("Live status: {0}".format(self.is_live))
        if self.is_live:
            setup.LIVE = True

    # Before tests are run live a resource group needs to be created
    # Note that when tests are run in live mode it is best to run one test at a time.
    @recorded_by_proxy
    def test_create_delete_account(self):
        print("Starting test_create_delete_account")
        account_name = self.get_resource_name(setup.TEST_ACC_1 + "-")
        account_list = self.client.accounts.list(setup.TEST_RG)
        account_list_lenght = len(list(account_list))

        print("create_delete_account {0}".format(account_name))
        account = create_account(self.client, setup.TEST_RG, account_name)
        assert account.name == account_name

        account_list = self.client.accounts.list(setup.TEST_RG)
        assert len(list(account_list)) == account_list_lenght + 1

        delete_account(self.client, setup.TEST_RG, account_name)
        account_list = self.client.accounts.list(setup.TEST_RG)
        assert len(list(account_list)) == account_list_lenght
        print("Finished with test_create_delete_account")

    @recorded_by_proxy
    def test_list_accounts(self):
        print("Starting test_list_accounts")
        account_name1 = self.get_resource_name(setup.TEST_ACC_1 + "-")
        account_name2 = self.get_resource_name(setup.TEST_ACC_2 + "-")
        account_list = self.client.accounts.list(setup.TEST_RG)
        account_list_lenght = len(list(account_list))

        create_account(self.client, setup.TEST_RG, account_name1)
        create_account(self.client, setup.TEST_RG, account_name2)
        accounts = [account_name1, account_name2]

        account_list = self.client.accounts.list(setup.TEST_RG)
        assert len(list(account_list)) == account_list_lenght + 2
        idx = 0
        for account in account_list:
            assert account.name == accounts[idx]
            idx += 1

        delete_account(self.client, setup.TEST_RG, account_name1)
        delete_account(self.client, setup.TEST_RG, account_name2)
        print("Finished with test_list_accounts")

    @recorded_by_proxy
    def test_get_account_by_name(self):
        print("Starting test_get_account_by_name")
        account_name1 = self.get_resource_name(setup.TEST_ACC_1 + "-")
        create_account(self.client, setup.TEST_RG, account_name1)

        account = self.client.accounts.get(setup.TEST_RG, account_name1)
        assert account.name == account_name1

        delete_account(self.client, setup.TEST_RG, account_name1)
        print("Finished with test_get_account_by_name")

    @recorded_by_proxy
    def test_patch_account(self):
        print("Starting test_patch_account")
        account_name1 = self.get_resource_name(setup.TEST_ACC_1 + "-")
        create_account(self.client, setup.TEST_RG, account_name1)

        tag = {"Tag1": "Value2"}
        netapp_account_patch = NetAppAccountPatch(tags=tag)

        account = self.client.accounts.begin_update(setup.TEST_RG, account_name1, netapp_account_patch).result()
        assert account.tags["Tag1"] == "Value2"

        delete_account(self.client, setup.TEST_RG, account_name1)
        print("Finished with test_patch_account")



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
# pylint: disable=wrong-import-position

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from ._patch import *  # pylint: disable=unused-wildcard-import

from ._operations import Operations  # type: ignore
from ._operations import NetAppResourceQuotaLimitsAccountOperations  # type: ignore
from ._operations import VolumeGroupsOperations  # type: ignore
from ._operations import BackupsOperations  # type: ignore
from ._operations import VolumesOperations  # type: ignore
from ._operations import SnapshotsOperations  # type: ignore
from ._operations import SnapshotPoliciesOperations  # type: ignore
from ._operations import BackupPoliciesOperations  # type: ignore
from ._operations import VolumeQuotaRulesOperations  # type: ignore
from ._operations import RansomwareReportsOperations  # type: ignore
from ._operations import BackupVaultsOperations  # type: ignore
from ._operations import BucketsOperations  # type: ignore
from ._operations import CachesOperations  # type: ignore
from ._operations import ElasticAccountsOperations  # type: ignore
from ._operations import ElasticCapacityPoolsOperations  # type: ignore
from ._operations import ElasticVolumesOperations  # type: ignore
from ._operations import ElasticSnapshotsOperations  # type: ignore
from ._operations import ElasticSnapshotPoliciesOperations  # type: ignore
from ._operations import ElasticBackupVaultsOperations  # type: ignore
from ._operations import ElasticBackupPoliciesOperations  # type: ignore
from ._operations import ElasticBackupsOperations  # type: ignore
from ._operations import ActiveDirectoryConfigsOperations  # type: ignore
from ._operations import NetAppResourceQuotaLimitsOperations  # type: ignore
from ._operations import NetAppResourceRegionInfosOperations  # type: ignore
from ._operations import AccountsOperations  # type: ignore
from ._operations import BackupsUnderAccountOperations  # type: ignore
from ._operations import PoolsOperations  # type: ignore
from ._operations import BackupsUnderBackupVaultOperations  # type: ignore
from ._operations import BackupsUnderVolumeOperations  # type: ignore
from ._operations import SubvolumesOperations  # type: ignore
from ._operations import NetAppResourceOperations  # type: ignore
from ._operations import NetAppResourceUsagesOperations  # type: ignore

from ._patch import __all__ as _patch_all
from ._patch import *
from ._patch import patch_sdk as _patch_sdk

__all__ = [
    "Operations",
    "NetAppResourceQuotaLimitsAccountOperations",
    "VolumeGroupsOperations",
    "BackupsOperations",
    "VolumesOperations",
    "SnapshotsOperations",
    "SnapshotPoliciesOperations",
    "BackupPoliciesOperations",
    "VolumeQuotaRulesOperations",
    "RansomwareReportsOperations",
    "BackupVaultsOperations",
    "BucketsOperations",
    "CachesOperations",
    "ElasticAccountsOperations",
    "ElasticCapacityPoolsOperations",
    "ElasticVolumesOperations",
    "ElasticSnapshotsOperations",
    "ElasticSnapshotPoliciesOperations",
    "ElasticBackupVaultsOperations",
    "ElasticBackupPoliciesOperations",
    "ElasticBackupsOperations",
    "ActiveDirectoryConfigsOperations",
    "NetAppResourceQuotaLimitsOperations",
    "NetAppResourceRegionInfosOperations",
    "AccountsOperations",
    "BackupsUnderAccountOperations",
    "PoolsOperations",
    "BackupsUnderBackupVaultOperations",
    "BackupsUnderVolumeOperations",
    "SubvolumesOperations",
    "NetAppResourceOperations",
    "NetAppResourceUsagesOperations",
]
__all__.extend([p for p in _patch_all if p not in __all__])  # pyright: ignore
_patch_sdk()



import random
import string
import time
from azure.mgmt.resource.resources import ResourceManagementClient
from devtools_testutils import AzureMgmtRecordedTestCase, recorded_by_proxy, set_bodiless_matcher
from azure.mgmt.netapp.models import Volume, Snapshot
from test_volume import create_volume, wait_for_volume, delete_volume, create_virtual_network
from test_pool import delete_pool
from test_account import delete_account
import setup
import azure.mgmt.netapp.models


def create_snapshot(
    client,
    rg=setup.TEST_RG,
    account_name=setup.TEST_ACC_1,
    pool_name=setup.TEST_POOL_1,
    volume_name=setup.TEST_VOL_1,
    snapshot_name=setup.TEST_SNAPSHOT_1,
    location=setup.LOCATION,
    vnet=setup.PERMA_VNET,
    snapshot_only=False,
):
    if not snapshot_only:
        volume = create_volume(
            client, rg, account_name, pool_name, volume_name, vnet=setup.PERMA_VNET, volume_only=True
        )
    else:
        # we need to get the volume id if we didn't just create it
        volume = client.volumes.get(rg, account_name, pool_name, volume_name)
    # snapshot_properties = azure.mgmt.netapp.models.SnapshotProperties(file_system_id=volume.file_system_id)
    body = Snapshot(location=location)
    client.snapshots.begin_create(rg, account_name, pool_name, volume_name, snapshot_name, body).result()
    snapshot = wait_for_snapshot(client, rg, account_name, pool_name, volume_name, snapshot_name)
    return snapshot


def delete_snapshot(client, rg, account_name, pool_name, volume_name, snapshot_name):
    client.snapshots.begin_delete(rg, account_name, pool_name, volume_name, snapshot_name).wait()
    wait_for_no_snapshot(client, rg, account_name, pool_name, volume_name, snapshot_name)


def wait_for_no_snapshot(client, rg, account_name, pool_name, volume_name, snapshot_name):
    retry = 0
    while retry < 60:
        retry += 1
        try:
            snapshot = client.snapshots.get(rg, account_name, pool_name, volume_name, snapshot_name)
            print("\tGot snapshot. Still waiting. state = " + snapshot.provisioning_state)
        except:
            # not found is an exception case (status code 200 expected)
            # but is what we are waiting for
            break
        if setup.LIVE:
            time.sleep(3)
    if retry == 60:
        raise Exception("Timeout when waiting for no snapshot")


def wait_for_snapshot(client, rg, account_name, pool_name, volume_name, snapshot_name):
    retry = 0
    while retry < 100:
        retry += 1
        snapshot = client.snapshots.get(rg, account_name, pool_name, volume_name, snapshot_name)
        if snapshot.provisioning_state == "Succeeded":
            break
        if snapshot.provisioning_state == "Failed":
            print("\t Wait for snapshot. Snapshot is in a failed state.")
            break
        if setup.LIVE:
            time.sleep(3)
    if retry == 100:
        raise Exception("Timeout when waiting for snapshot")
    return snapshot


class TestNetAppSnapshot(AzureMgmtRecordedTestCase):

    def setup_method(self, method):
        self.client = self.create_mgmt_client(azure.mgmt.netapp.NetAppManagementClient)
        if self.is_live:
            setup.LIVE = True

    # Note that when tests are run in live mode it is best to run one test at a time.
    @recorded_by_proxy
    def test_create_delete_snapshot(self):
        print("Starting test_create_delete_snapshot")
        set_bodiless_matcher()
        volumeName1 = self.get_resource_name(setup.TEST_VOL_1 + "-")
        snapshotname = self.get_resource_name(setup.TEST_SNAPSHOT_1 + "-")
        try:
            create_snapshot(
                self.client,
                setup.TEST_RG,
                setup.PERMA_ACCOUNT,
                setup.PERMA_POOL,
                volumeName1,
                snapshotname,
                setup.LOCATION,
                vnet=setup.PERMA_VNET,
            )

            snapshot_list = self.client.snapshots.list(
                setup.TEST_RG, setup.PERMA_ACCOUNT, setup.PERMA_POOL, volumeName1
            )
            assert len(list(snapshot_list)) == 1

            delete_snapshot(
                self.client, setup.TEST_RG, setup.PERMA_ACCOUNT, setup.PERMA_POOL, volumeName1, snapshotname
            )
            snapshot_list = self.client.snapshots.list(
                setup.TEST_RG, setup.PERMA_ACCOUNT, setup.PERMA_POOL, volumeName1
            )
            assert len(list(snapshot_list)) == 0
        finally:
            delete_volume(self.client, setup.TEST_RG, setup.PERMA_ACCOUNT, setup.PERMA_POOL, volumeName1)

        print("Finished with test_create_delete_snapshot")

    @recorded_by_proxy
    def test_list_snapshots(self):
        print("Starting test_list_snapshots")
        set_bodiless_matcher()
        volumeName1 = self.get_resource_name(setup.TEST_VOL_1 + "-")
        snapshotname = self.get_resource_name(setup.TEST_SNAPSHOT_1 + "-")
        snapshotname2 = self.get_resource_name(setup.TEST_SNAPSHOT_2 + "-")

        try:
            create_snapshot(
                self.client,
                setup.TEST_RG,
                setup.PERMA_ACCOUNT,
                setup.PERMA_POOL,
                volumeName1,
                snapshotname,
                setup.LOCATION,
                vnet=setup.PERMA_VNET,
            )
            create_snapshot(
                self.client,
                setup.TEST_RG,
                setup.PERMA_ACCOUNT,
                setup.PERMA_POOL,
                volumeName1,
                snapshotname2,
                setup.LOCATION,
                snapshot_only=True,
            )
            snapshots = [snapshotname, snapshotname2]

            snapshot_list = self.client.snapshots.list(
                setup.TEST_RG, setup.PERMA_ACCOUNT, setup.PERMA_POOL, volumeName1
            )
            assert len(list(snapshot_list)) == 2
            idx = 0
            for snapshot in snapshot_list:
                assert snapshot.name == snapshots[idx]
                idx += 1
        finally:
            delete_snapshot(
                self.client, setup.TEST_RG, setup.PERMA_ACCOUNT, setup.PERMA_POOL, volumeName1, snapshotname
            )
            delete_snapshot(
                self.client, setup.TEST_RG, setup.PERMA_ACCOUNT, setup.PERMA_POOL, volumeName1, snapshotname2
            )
            delete_volume(self.client, setup.TEST_RG, setup.PERMA_ACCOUNT, setup.PERMA_POOL, volumeName1)

        print("Finished with test_list_snapshots")

    @recorded_by_proxy
    def test_get_snapshot_by_name(self):
        print("Starting test_get_snapshot_by_name")
        set_bodiless_matcher()
        volumeName1 = self.get_resource_name(setup.TEST_VOL_1 + "-")
        snapshotname = self.get_resource_name(setup.TEST_SNAPSHOT_1 + "-")

        try:
            create_snapshot(
                self.client,
                setup.TEST_RG,
                setup.PERMA_ACCOUNT,
                setup.PERMA_POOL,
                volumeName1,
                snapshotname,
                setup.LOCATION,
                vnet=setup.PERMA_VNET,
            )

            snapshot = self.client.snapshots.get(
                setup.TEST_RG, setup.PERMA_ACCOUNT, setup.PERMA_POOL, volumeName1, snapshotname
            )
            assert (
                snapshot.name == setup.PERMA_ACCOUNT + "/" + setup.PERMA_POOL + "/" + volumeName1 + "/" + snapshotname
            )
        finally:
            delete_snapshot(
                self.client, setup.TEST_RG, setup.PERMA_ACCOUNT, setup.PERMA_POOL, volumeName1, snapshotname
            )
            delete_volume(self.client, setup.TEST_RG, setup.PERMA_ACCOUNT, setup.PERMA_POOL, volumeName1)

        print("Finished with test_get_snapshot_by_name")



# pylint: disable=line-too-long,useless-suppression
from devtools_testutils import AzureMgmtRecordedTestCase, recorded_by_proxy
import setup
import pytest
import azure.mgmt.netapp.models


class TestNetAppResourceOperations(AzureMgmtRecordedTestCase):

    def setup_method(self, method):
        self.client = self.create_mgmt_client(azure.mgmt.netapp.NetAppManagementClient)

    # Before tests are run live a resource group needs to be created along with vnet and subnet
    # Note that when tests are run in live mode it is best to run one test at a time.
    @pytest.mark.skip(reason="Skipping this test for service side issue re-enable when fixed")
    @recorded_by_proxy
    def test_query_network_sibling_set(self):

        print("Starting test_query_network_sibling_set")
        try:
            self.client.net_app_resource.query_network_sibling_set(
                setup.LOCATION,
                "9760acf5-4638-11e7-9bdb-020073ca3333",
                "/subscriptions/00000000-0000-0000-0000-000000000000/resourceGroups/sdk-python-tests-rg-tmp/providers/Microsoft.Network/virtualNetworks/testVnet/subnets/sdk-python-tests-subnet",
            )
        except Exception as e:
            assert str(e).startswith("(NoNicsFoundForSubnet)")
        print("Finished with test_query_network_sibling_set")

    @pytest.mark.skip(reason="Skipping this test for service side issue re-enable when fixed")
    @recorded_by_proxy
    def test_begin_update_network_sibling_set(self):

        print("Starting test_begin_update_network_sibling_set")
        try:
            self.client.net_app_resource.begin_update_network_sibling_set(
                setup.LOCATION,
                "9760acf5-4638-11e7-9bdb-020073ca3333",
                "/subscriptions/00000000-0000-0000-0000-000000000000/resourceGroups/sdk-python-tests-rg-tmp/providers/Microsoft.Network/virtualNetworks/testVnet/subnets/sdk-python-tests-subnet",
                "herpiderp",
                "Standard",
            )
        except Exception as e:
            assert str(e).startswith("(NetworkSiblingSetChanged)")
        print("Finished with test_begin_update_network_sibling_set")



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
# pylint: disable=wrong-import-position

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from ._patch import *  # pylint: disable=unused-wildcard-import


from ._models import (  # type: ignore
    AccountEncryption,
    AccountProperties,
    AccountPropertiesPatch,
    ActiveDirectory,
    ActiveDirectoryConfig,
    ActiveDirectoryConfigProperties,
    ActiveDirectoryConfigUpdate,
    ActiveDirectoryConfigUpdateProperties,
    AuthorizeRequest,
    AzureKeyVaultDetails,
    Backup,
    BackupPatch,
    BackupPatchProperties,
    BackupPolicy,
    BackupPolicyPatch,
    BackupPolicyProperties,
    BackupProperties,
    BackupRestoreFiles,
    BackupStatus,
    BackupVault,
    BackupVaultPatch,
    BackupVaultProperties,
    BackupsMigrationRequest,
    BindPasswordAkvConfig,
    BindPasswordAkvConfigPatch,
    BreakFileLocksRequest,
    BreakReplicationRequest,
    Bucket,
    BucketCredentialsExpiry,
    BucketGenerateCredentials,
    BucketPatch,
    BucketPatchProperties,
    BucketProperties,
    BucketServerPatchProperties,
    BucketServerProperties,
    Cache,
    CacheMountTargetProperties,
    CacheProperties,
    CachePropertiesExportPolicy,
    CacheUpdate,
    CacheUpdateProperties,
    CapacityPool,
    CapacityPoolPatch,
    CertificateAkvDetails,
    ChangeKeyVault,
    ChangeZoneRequest,
    CheckAvailabilityResponse,
    CheckElasticResourceAvailabilityResponse,
    CheckElasticVolumeFilePathAvailabilityRequest,
    CifsUser,
    ClusterPeerCommandResponse,
    ClusterPeerCommandResponseProperties,
    CredentialsAkvDetails,
    DailySchedule,
    DestinationReplication,
    Dimension,
    ElasticAccount,
    ElasticAccountProperties,
    ElasticAccountUpdate,
    ElasticAccountUpdateProperties,
    ElasticBackup,
    ElasticBackupPolicy,
    ElasticBackupPolicyProperties,
    ElasticBackupPolicyUpdate,
    ElasticBackupPolicyUpdateProperties,
    ElasticBackupProperties,
    ElasticBackupVault,
    ElasticBackupVaultProperties,
    ElasticBackupVaultUpdate,
    ElasticCapacityPool,
    ElasticCapacityPoolProperties,
    ElasticCapacityPoolUpdate,
    ElasticCapacityPoolUpdateProperties,
    ElasticEncryption,
    ElasticEncryptionConfiguration,
    ElasticEncryptionIdentity,
    ElasticExportPolicy,
    ElasticExportPolicyRule,
    ElasticKeyVaultProperties,
    ElasticMountTargetProperties,
    ElasticSmbPatchProperties,
    ElasticSmbProperties,
    ElasticSnapshot,
    ElasticSnapshotPolicy,
    ElasticSnapshotPolicyDailySchedule,
    ElasticSnapshotPolicyHourlySchedule,
    ElasticSnapshotPolicyMonthlySchedule,
    ElasticSnapshotPolicyProperties,
    ElasticSnapshotPolicyUpdate,
    ElasticSnapshotPolicyUpdateProperties,
    ElasticSnapshotPolicyWeeklySchedule,
    ElasticSnapshotProperties,
    ElasticVolume,
    ElasticVolumeBackupProperties,
    ElasticVolumeDataProtectionPatchProperties,
    ElasticVolumeDataProtectionProperties,
    ElasticVolumeProperties,
    ElasticVolumeRevert,
    ElasticVolumeSnapshotProperties,
    ElasticVolumeUpdate,
    ElasticVolumeUpdateProperties,
    EncryptionIdentity,
    EncryptionTransitionRequest,
    EntraIdAkvConfig,
    EntraIdAkvConfigPatch,
    EntraIdConfig,
    EntraIdConfigPatch,
    ErrorAdditionalInfo,
    ErrorDetail,
    ErrorResponse,
    ExportPolicyRule,
    FilePathAvailabilityRequest,
    FileSystemUser,
    GetGroupIdListForLDAPUserRequest,
    GetGroupIdListForLDAPUserResponse,
    GetKeyVaultStatusResponse,
    GetKeyVaultStatusResponseProperties,
    HourlySchedule,
    KeyVaultPrivateEndpoint,
    KeyVaultProperties,
    LdapConfiguration,
    LdapConfigurationPatch,
    LdapSearchScopeOpt,
    ListQuotaReportResponse,
    ListQuotaReportResult,
    ListReplicationsRequest,
    LogSpecification,
    ManagedServiceIdentity,
    MetricSpecification,
    MonthlySchedule,
    MountTargetProperties,
    NetAppAccount,
    NetAppAccountPatch,
    NetworkSiblingSet,
    NfsUser,
    NicInfo,
    Operation,
    OperationDisplay,
    OperationProperties,
    OriginClusterInformation,
    PeerClusterForVolumeMigrationRequest,
    PeeringPassphrases,
    PlacementKeyValuePairs,
    PoolChangeRequest,
    PoolPatchProperties,
    PoolProperties,
    ProxyResource,
    QueryNetworkSiblingSetRequest,
    QuotaAvailabilityRequest,
    QuotaItem,
    QuotaItemProperties,
    QuotaReport,
    QuotaReportFilterRequest,
    RansomwareProtectionPatchSettings,
    RansomwareProtectionSettings,
    RansomwareReport,
    RansomwareReportProperties,
    RansomwareSuspects,
    RansomwareSuspectsClearRequest,
    ReestablishReplicationRequest,
    RegionInfo,
    RegionInfoAvailabilityZoneMappingsItem,
    RegionInfoResource,
    RelocateVolumeRequest,
    RemotePath,
    Replication,
    ReplicationObject,
    ReplicationStatus,
    Resource,
    ResourceNameAvailabilityRequest,
    RestoreStatus,
    SecretPassword,
    SecretPasswordIdentity,
    SecretPasswordKeyVaultProperties,
    ServiceSpecification,
    SmbSettings,
    Snapshot,
    SnapshotPatch,
    SnapshotPolicy,
    SnapshotPolicyPatch,
    SnapshotPolicyProperties,
    SnapshotPolicyVolumeList,
    SnapshotProperties,
    SnapshotRestoreFiles,
    SubvolumeInfo,
    SubvolumeModel,
    SubvolumeModelProperties,
    SubvolumePatchParams,
    SubvolumePatchRequest,
    SubvolumeProperties,
    SuspectFile,
    SvmPeerCommandResponse,
    SvmPeerCommandResponseProperties,
    SystemData,
    TrackedResource,
    UpdateNetworkSiblingSetRequest,
    UsageName,
    UsageProperties,
    UsageResult,
    UserAssignedIdentity,
    Volume,
    VolumeBackupProperties,
    VolumeBackups,
    VolumeGroup,
    VolumeGroupDetails,
    VolumeGroupListProperties,
    VolumeGroupMetaData,
    VolumeGroupProperties,
    VolumeGroupVolumeProperties,
    VolumePatch,
    VolumePatchProperties,
    VolumePatchPropertiesDataProtection,
    VolumePatchPropertiesExportPolicy,
    VolumeProperties,
    VolumePropertiesDataProtection,
    VolumePropertiesExportPolicy,
    VolumeQuotaRule,
    VolumeQuotaRulePatch,
    VolumeQuotaRulesProperties,
    VolumeRelocationProperties,
    VolumeRevert,
    VolumeSnapshotProperties,
    WeeklySchedule,
)

from ._enums import (  # type: ignore
    AcceptGrowCapacityPoolForShortTermCloneSplit,
    ActiveDirectoryStatus,
    ActualRansomwareProtectionState,
    ApplicationType,
    AvsDataStore,
    BackupType,
    BindAuthenticationLevel,
    BreakthroughMode,
    BucketPatchPermissions,
    BucketPermissions,
    CacheFileAccessLogs,
    CacheLifeCycleState,
    CacheProvisioningState,
    CheckElasticResourceAvailabilityReason,
    CheckElasticResourceAvailabilityStatus,
    CheckNameResourceTypes,
    CheckQuotaNameResourceTypes,
    ChownMode,
    CifsChangeNotifyState,
    CoolAccessRetrievalPolicy,
    CoolAccessTieringPolicy,
    CreatedByType,
    CredentialsStatus,
    DayOfWeek,
    DesiredRansomwareProtectionState,
    ElasticBackupPolicyState,
    ElasticBackupType,
    ElasticKeyVaultStatus,
    ElasticNfsv3Access,
    ElasticNfsv4Access,
    ElasticPoolEncryptionKeySource,
    ElasticProtocolType,
    ElasticResourceAvailabilityStatus,
    ElasticRootAccess,
    ElasticServiceLevel,
    ElasticSmbEncryption,
    ElasticUnixAccessRule,
    ElasticVolumePolicyEnforcement,
    ElasticVolumeRestorationState,
    EnableSubvolumes,
    EnableWriteBackState,
    EncryptionKeySource,
    EncryptionState,
    EncryptionType,
    EndpointType,
    Exclude,
    ExternalReplicationSetupStatus,
    FileAccessLogs,
    GlobalFileLockingState,
    InAvailabilityReasonType,
    KerberosState,
    KeySource,
    KeyVaultStatus,
    LargeVolumeType,
    LdapServerType,
    LdapState,
    ManagedServiceIdentityType,
    MetricAggregationType,
    MirrorState,
    MultiAdStatus,
    NetAppProvisioningState,
    NetworkFeatures,
    NetworkSiblingSetProvisioningState,
    OnCertificateConflictAction,
    PolicyStatus,
    ProtocolTypes,
    QosType,
    QuotaType,
    RansomwareReportSeverity,
    RansomwareReportState,
    RansomwareSuspectResolution,
    RegionStorageToNetworkProximity,
    ReplicationMirrorState,
    ReplicationSchedule,
    ReplicationType,
    SecurityStyle,
    ServiceLevel,
    SmbAccessBasedEnumeration,
    SmbEncryptionState,
    SmbNonBrowsable,
    SnapshotDirectoryVisibility,
    SnapshotUsage,
    VolumeBackupRelationshipStatus,
    VolumeLanguage,
    VolumeReplicationRelationshipStatus,
    VolumeRestoreRelationshipStatus,
    VolumeSize,
    VolumeStorageToNetworkProximity,
)
from ._patch import __all__ as _patch_all
from ._patch import *
from ._patch import patch_sdk as _patch_sdk

__all__ = [
    "AccountEncryption",
    "AccountProperties",
    "AccountPropertiesPatch",
    "ActiveDirectory",
    "ActiveDirectoryConfig",
    "ActiveDirectoryConfigProperties",
    "ActiveDirectoryConfigUpdate",
    "ActiveDirectoryConfigUpdateProperties",
    "AuthorizeRequest",
    "AzureKeyVaultDetails",
    "Backup",
    "BackupPatch",
    "BackupPatchProperties",
    "BackupPolicy",
    "BackupPolicyPatch",
    "BackupPolicyProperties",
    "BackupProperties",
    "BackupRestoreFiles",
    "BackupStatus",
    "BackupVault",
    "BackupVaultPatch",
    "BackupVaultProperties",
    "BackupsMigrationRequest",
    "BindPasswordAkvConfig",
    "BindPasswordAkvConfigPatch",
    "BreakFileLocksRequest",
    "BreakReplicationRequest",
    "Bucket",
    "BucketCredentialsExpiry",
    "BucketGenerateCredentials",
    "BucketPatch",
    "BucketPatchProperties",
    "BucketProperties",
    "BucketServerPatchProperties",
    "BucketServerProperties",
    "Cache",
    "CacheMountTargetProperties",
    "CacheProperties",
    "CachePropertiesExportPolicy",
    "CacheUpdate",
    "CacheUpdateProperties",
    "CapacityPool",
    "CapacityPoolPatch",
    "CertificateAkvDetails",
    "ChangeKeyVault",
    "ChangeZoneRequest",
    "CheckAvailabilityResponse",
    "CheckElasticResourceAvailabilityResponse",
    "CheckElasticVolumeFilePathAvailabilityRequest",
    "CifsUser",
    "ClusterPeerCommandResponse",
    "ClusterPeerCommandResponseProperties",
    "CredentialsAkvDetails",
    "DailySchedule",
    "DestinationReplication",
    "Dimension",
    "ElasticAccount",
    "ElasticAccountProperties",
    "ElasticAccountUpdate",
    "ElasticAccountUpdateProperties",
    "ElasticBackup",
    "ElasticBackupPolicy",
    "ElasticBackupPolicyProperties",
    "ElasticBackupPolicyUpdate",
    "ElasticBackupPolicyUpdateProperties",
    "ElasticBackupProperties",
    "ElasticBackupVault",
    "ElasticBackupVaultProperties",
    "ElasticBackupVaultUpdate",
    "ElasticCapacityPool",
    "ElasticCapacityPoolProperties",
    "ElasticCapacityPoolUpdate",
    "ElasticCapacityPoolUpdateProperties",
    "ElasticEncryption",
    "ElasticEncryptionConfiguration",
    "ElasticEncryptionIdentity",
    "ElasticExportPolicy",
    "ElasticExportPolicyRule",
    "ElasticKeyVaultProperties",
    "ElasticMountTargetProperties",
    "ElasticSmbPatchProperties",
    "ElasticSmbProperties",
    "ElasticSnapshot",
    "ElasticSnapshotPolicy",
    "ElasticSnapshotPolicyDailySchedule",
    "ElasticSnapshotPolicyHourlySchedule",
    "ElasticSnapshotPolicyMonthlySchedule",
    "ElasticSnapshotPolicyProperties",
    "ElasticSnapshotPolicyUpdate",
    "ElasticSnapshotPolicyUpdateProperties",
    "ElasticSnapshotPolicyWeeklySchedule",
    "ElasticSnapshotProperties",
    "ElasticVolume",
    "ElasticVolumeBackupProperties",
    "ElasticVolumeDataProtectionPatchProperties",
    "ElasticVolumeDataProtectionProperties",
    "ElasticVolumeProperties",
    "ElasticVolumeRevert",
    "ElasticVolumeSnapshotProperties",
    "ElasticVolumeUpdate",
    "ElasticVolumeUpdateProperties",
    "EncryptionIdentity",
    "EncryptionTransitionRequest",
    "EntraIdAkvConfig",
    "EntraIdAkvConfigPatch",
    "EntraIdConfig",
    "EntraIdConfigPatch",
    "ErrorAdditionalInfo",
    "ErrorDetail",
    "ErrorResponse",
    "ExportPolicyRule",
    "FilePathAvailabilityRequest",
    "FileSystemUser",
    "GetGroupIdListForLDAPUserRequest",
    "GetGroupIdListForLDAPUserResponse",
    "GetKeyVaultStatusResponse",
    "GetKeyVaultStatusResponseProperties",
    "HourlySchedule",
    "KeyVaultPrivateEndpoint",
    "KeyVaultProperties",
    "LdapConfiguration",
    "LdapConfigurationPatch",
    "LdapSearchScopeOpt",
    "ListQuotaReportResponse",
    "ListQuotaReportResult",
    "ListReplicationsRequest",
    "LogSpecification",
    "ManagedServiceIdentity",
    "MetricSpecification",
    "MonthlySchedule",
    "MountTargetProperties",
    "NetAppAccount",
    "NetAppAccountPatch",
    "NetworkSiblingSet",
    "NfsUser",
    "NicInfo",
    "Operation",
    "OperationDisplay",
    "OperationProperties",
    "OriginClusterInformation",
    "PeerClusterForVolumeMigrationRequest",
    "PeeringPassphrases",
    "PlacementKeyValuePairs",
    "PoolChangeRequest",
    "PoolPatchProperties",
    "PoolProperties",
    "ProxyResource",
    "QueryNetworkSiblingSetRequest",
    "QuotaAvailabilityRequest",
    "QuotaItem",
    "QuotaItemProperties",
    "QuotaReport",
    "QuotaReportFilterRequest",
    "RansomwareProtectionPatchSettings",
    "RansomwareProtectionSettings",
    "RansomwareReport",
    "RansomwareReportProperties",
    "RansomwareSuspects",
    "RansomwareSuspectsClearRequest",
    "ReestablishReplicationRequest",
    "RegionInfo",
    "RegionInfoAvailabilityZoneMappingsItem",
    "RegionInfoResource",
    "RelocateVolumeRequest",
    "RemotePath",
    "Replication",
    "ReplicationObject",
    "ReplicationStatus",
    "Resource",
    "ResourceNameAvailabilityRequest",
    "RestoreStatus",
    "SecretPassword",
    "SecretPasswordIdentity",
    "SecretPasswordKeyVaultProperties",
    "ServiceSpecification",
    "SmbSettings",
    "Snapshot",
    "SnapshotPatch",
    "SnapshotPolicy",
    "SnapshotPolicyPatch",
    "SnapshotPolicyProperties",
    "SnapshotPolicyVolumeList",
    "SnapshotProperties",
    "SnapshotRestoreFiles",
    "SubvolumeInfo",
    "SubvolumeModel",
    "SubvolumeModelProperties",
    "SubvolumePatchParams",
    "SubvolumePatchRequest",
    "SubvolumeProperties",
    "SuspectFile",
    "SvmPeerCommandResponse",
    "SvmPeerCommandResponseProperties",
    "SystemData",
    "TrackedResource",
    "UpdateNetworkSiblingSetRequest",
    "UsageName",
    "UsageProperties",
    "UsageResult",
    "UserAssignedIdentity",
    "Volume",
    "VolumeBackupProperties",
    "VolumeBackups",
    "VolumeGroup",
    "VolumeGroupDetails",
    "VolumeGroupListProperties",
    "VolumeGroupMetaData",
    "VolumeGroupProperties",
    "VolumeGroupVolumeProperties",
    "VolumePatch",
    "VolumePatchProperties",
    "VolumePatchPropertiesDataProtection",
    "VolumePatchPropertiesExportPolicy",
    "VolumeProperties",
    "VolumePropertiesDataProtection",
    "VolumePropertiesExportPolicy",
    "VolumeQuotaRule",
    "VolumeQuotaRulePatch",
    "VolumeQuotaRulesProperties",
    "VolumeRelocationProperties",
    "VolumeRevert",
    "VolumeSnapshotProperties",
    "WeeklySchedule",
    "AcceptGrowCapacityPoolForShortTermCloneSplit",
    "ActiveDirectoryStatus",
    "ActualRansomwareProtectionState",
    "ApplicationType",
    "AvsDataStore",
    "BackupType",
    "BindAuthenticationLevel",
    "BreakthroughMode",
    "BucketPatchPermissions",
    "BucketPermissions",
    "CacheFileAccessLogs",
    "CacheLifeCycleState",
    "CacheProvisioningState",
    "CheckElasticResourceAvailabilityReason",
    "CheckElasticResourceAvailabilityStatus",
    "CheckNameResourceTypes",
    "CheckQuotaNameResourceTypes",
    "ChownMode",
    "CifsChangeNotifyState",
    "CoolAccessRetrievalPolicy",
    "CoolAccessTieringPolicy",
    "CreatedByType",
    "CredentialsStatus",
    "DayOfWeek",
    "DesiredRansomwareProtectionState",
    "ElasticBackupPolicyState",
    "ElasticBackupType",
    "ElasticKeyVaultStatus",
    "ElasticNfsv3Access",
    "ElasticNfsv4Access",
    "ElasticPoolEncryptionKeySource",
    "ElasticProtocolType",
    "ElasticResourceAvailabilityStatus",
    "ElasticRootAccess",
    "ElasticServiceLevel",
    "ElasticSmbEncryption",
    "ElasticUnixAccessRule",
    "ElasticVolumePolicyEnforcement",
    "ElasticVolumeRestorationState",
    "EnableSubvolumes",
    "EnableWriteBackState",
    "EncryptionKeySource",
    "EncryptionState",
    "EncryptionType",
    "EndpointType",
    "Exclude",
    "ExternalReplicationSetupStatus",
    "FileAccessLogs",
    "GlobalFileLockingState",
    "InAvailabilityReasonType",
    "KerberosState",
    "KeySource",
    "KeyVaultStatus",
    "LargeVolumeType",
    "LdapServerType",
    "LdapState",
    "ManagedServiceIdentityType",
    "MetricAggregationType",
    "MirrorState",
    "MultiAdStatus",
    "NetAppProvisioningState",
    "NetworkFeatures",
    "NetworkSiblingSetProvisioningState",
    "OnCertificateConflictAction",
    "PolicyStatus",
    "ProtocolTypes",
    "QosType",
    "QuotaType",
    "RansomwareReportSeverity",
    "RansomwareReportState",
    "RansomwareSuspectResolution",
    "RegionStorageToNetworkProximity",
    "ReplicationMirrorState",
    "ReplicationSchedule",
    "ReplicationType",
    "SecurityStyle",
    "ServiceLevel",
    "SmbAccessBasedEnumeration",
    "SmbEncryptionState",
    "SmbNonBrowsable",
    "SnapshotDirectoryVisibility",
    "SnapshotUsage",
    "VolumeBackupRelationshipStatus",
    "VolumeLanguage",
    "VolumeReplicationRelationshipStatus",
    "VolumeRestoreRelationshipStatus",
    "VolumeSize",
    "VolumeStorageToNetworkProximity",
]
__all__.extend([p for p in _patch_all if p not in __all__])  # pyright: ignore
_patch_sdk()



import time
from devtools_testutils import AzureMgmtRecordedTestCase, recorded_by_proxy, set_bodiless_matcher
from azure.mgmt.netapp.models import (
    SnapshotPolicy,
    SnapshotPolicyPatch,
    SnapshotPolicyProperties,
    HourlySchedule,
    DailySchedule,
    VolumeSnapshotProperties,
    VolumePatchPropertiesDataProtection,
    VolumePatchProperties,
    VolumePatch,
)
from test_account import create_account, delete_account
from test_pool import delete_pool
from test_volume import create_volume, delete_volume, wait_for_volume
import setup
import azure.mgmt.netapp.models

setup.TEST_SNAPSHOT_POLICY_1 = "sdk-py-tests-snapshot-policy-1"
setup.TEST_SNAPSHOT_POLICY_2 = "sdk-py-tests-snapshot-policy-2"


def create_snapshot_policy(
    client,
    snapshot_policy_name,
    rg=setup.TEST_RG,
    account_name=setup.TEST_ACC_1,
    location=setup.LOCATION,
    snapshot_policy_only=False,
):
    if not snapshot_policy_only:
        create_account(client, rg, account_name, location)

    snapshot_policy_body = SnapshotPolicy(
        location=location,
        hourly_schedule=HourlySchedule(snapshots_to_keep=1, minute=50),
        daily_schedule={},
        weekly_schedule={},
        monthly_schedule={},
        enabled=False,
    )

    snapshot_policy = client.snapshot_policies.create(rg, account_name, snapshot_policy_name, snapshot_policy_body)
    wait_for_snapshot_policy(client, rg, account_name, snapshot_policy_name)
    return snapshot_policy


def wait_for_snapshot_policy(client, rg, account_name, snapshot_policy_name):
    # a work around for the async nature of certain ARM processes
    retry = 0
    while retry < 60:
        retry += 1
        snapshot_policy = client.snapshot_policies.get(rg, account_name, snapshot_policy_name)
        if snapshot_policy.provisioning_state == "Succeeded":
            break
        if snapshot_policy.provisioning_state == "Failed":
            print("\t Wait for snapshotPolicy. SnapshotPolicy is in a failed state.")
            break
        if setup.LIVE:
            time.sleep(3)
    if retry == 60:
        raise Exception("Timeout when waiting for snapshot policy")


def delete_snapshot_policy(client, snapshot_policy_name, rg=setup.TEST_RG, account_name=setup.TEST_ACC_1):
    client.snapshot_policies.begin_delete(rg, account_name, snapshot_policy_name).wait()
    wait_for_no_snapshot_policy(client, rg, account_name, snapshot_policy_name)


def wait_for_no_snapshot_policy(client, rg, account_name, snapshot_policy_name):
    # a workaround for the async nature of certain ARM processes
    retry = 0
    while retry < 60:
        retry += 1
        try:
            client.snapshot_policies.get(rg, account_name, snapshot_policy_name)
        except:
            # not found is an exception case (status code 200 expected)
            # and is actually what we are waiting for
            break
        if setup.LIVE:
            time.sleep(3)
    if retry == 60:
        raise Exception("Timeout when waiting for no snapshot policy")


class TestNetAppSnapshotPolicy(AzureMgmtRecordedTestCase):

    def setup_method(self, method):
        self.client = self.create_mgmt_client(azure.mgmt.netapp.NetAppManagementClient)
        if self.is_live:
            setup.LIVE = True

    # Before tests are run live a resource group needs to be created along with vnet and subnet
    # Note that when tests are run in live mode it is best to run one test at a time.
    @recorded_by_proxy
    def test_create_delete_snapshot_policy(self):
        print("Starting test_create_delete_snapshot_policy")
        ACCOUNT1 = self.get_resource_name(setup.TEST_ACC_1 + "-")
        create_snapshot_policy(self.client, setup.TEST_SNAPSHOT_POLICY_1, account_name=ACCOUNT1)

        snapshot_policies_list = self.client.snapshot_policies.list(setup.TEST_RG, account_name=ACCOUNT1)
        assert len(list(snapshot_policies_list)) == 1

        delete_snapshot_policy(self.client, setup.TEST_SNAPSHOT_POLICY_1, account_name=ACCOUNT1)

        snapshot_policies_list = self.client.snapshot_policies.list(setup.TEST_RG, ACCOUNT1)
        assert len(list(snapshot_policies_list)) == 0

        delete_account(self.client, setup.TEST_RG, ACCOUNT1)
        print("Finished with test_create_delete_snapshot_policy")

    @recorded_by_proxy
    def test_list_snapshot_policies(self):
        print("Starting test_list_snapshot_policies")
        ACCOUNT1 = self.get_resource_name(setup.TEST_ACC_1 + "-")
        create_snapshot_policy(self.client, setup.TEST_SNAPSHOT_POLICY_1, account_name=ACCOUNT1)
        create_snapshot_policy(
            self.client, setup.TEST_SNAPSHOT_POLICY_2, account_name=ACCOUNT1, snapshot_policy_only=True
        )

        snapshot_policies_list = self.client.snapshot_policies.list(setup.TEST_RG, ACCOUNT1)
        assert len(list(snapshot_policies_list)) == 2
        snapshot_policies = [setup.TEST_SNAPSHOT_POLICY_1, setup.TEST_SNAPSHOT_POLICY_2]

        idx = 0
        for snapshot_policy in snapshot_policies_list:
            assert snapshot_policy.name == snapshot_policies[idx]
            idx += 1

        delete_snapshot_policy(self.client, setup.TEST_SNAPSHOT_POLICY_1, account_name=ACCOUNT1)
        delete_snapshot_policy(self.client, setup.TEST_SNAPSHOT_POLICY_2, account_name=ACCOUNT1)

        snapshot_policies_list = self.client.snapshot_policies.list(setup.TEST_RG, ACCOUNT1)
        assert len(list(snapshot_policies_list)) == 0

        delete_account(self.client, setup.TEST_RG, ACCOUNT1)
        print("Finished with test_list_snapshot_policies")

    @recorded_by_proxy
    def test_get_snapshot_policy_by_name(self):
        print("Starting test_get_snapshot_policy_by_name")
        ACCOUNT1 = self.get_resource_name(setup.TEST_ACC_1 + "-")
        create_snapshot_policy(self.client, setup.TEST_SNAPSHOT_POLICY_1, account_name=ACCOUNT1)

        snapshot_policy = self.client.snapshot_policies.get(setup.TEST_RG, ACCOUNT1, setup.TEST_SNAPSHOT_POLICY_1)
        assert snapshot_policy.name == ACCOUNT1 + "/" + setup.TEST_SNAPSHOT_POLICY_1

        delete_snapshot_policy(self.client, setup.TEST_SNAPSHOT_POLICY_1, account_name=ACCOUNT1)
        delete_account(self.client, setup.TEST_RG, ACCOUNT1)
        print("Finished with test_get_snapshot_policy_by_name")

    @recorded_by_proxy
    def test_update_snapshot_policies(self):
        print("Starting test_update_snapshot_policies")
        ACCOUNT1 = self.get_resource_name(setup.TEST_ACC_1 + "-")
        create_snapshot_policy(self.client, setup.TEST_SNAPSHOT_POLICY_1, account_name=ACCOUNT1)
        snapshot_policy_patchproperties = SnapshotPolicyProperties(
            # hourly_schedule={},
            daily_schedule=DailySchedule(snapshots_to_keep=1, minute=50, hour=1),
            # weekly_schedule={},
            # monthly_schedule={},
            enabled=False,
        )
        snapshot_policy_body = SnapshotPolicyPatch(properties=snapshot_policy_patchproperties)

        snapshot_policy = self.client.snapshot_policies.begin_update(
            setup.TEST_RG, ACCOUNT1, setup.TEST_SNAPSHOT_POLICY_1, snapshot_policy_body
        ).result()
        wait_for_snapshot_policy(self.client, setup.TEST_RG, ACCOUNT1, setup.TEST_SNAPSHOT_POLICY_1)
        assert snapshot_policy.daily_schedule.snapshots_to_keep == 1
        assert snapshot_policy.daily_schedule.hour == 1
        assert snapshot_policy.daily_schedule.minute == 50

        delete_snapshot_policy(self.client, setup.TEST_SNAPSHOT_POLICY_1, account_name=ACCOUNT1)
        delete_account(self.client, setup.TEST_RG, ACCOUNT1)
        print("Finished with test_update_snapshot_policies")

    @recorded_by_proxy
    def test_assign_snapshot_policy_to_volume(self):
        print("Starting test_assign_snapshot_policy_to_volume")
        set_bodiless_matcher()
        # create volume and snapshot policy
        ACCOUNT1 = self.get_resource_name(setup.TEST_ACC_1 + "-")
        volumeName1 = self.get_resource_name(setup.TEST_VOL_1 + "-")

        create_volume(
            self.client, setup.TEST_RG, ACCOUNT1, setup.TEST_POOL_1, volumeName1, setup.LOCATION, vnet=setup.PERMA_VNET
        )

        snapshot_policy = create_snapshot_policy(self.client, setup.TEST_SNAPSHOT_POLICY_1, account_name=ACCOUNT1)
        # assign the snapshot policy to the volume
        snapshot = VolumeSnapshotProperties(snapshot_policy_id=snapshot_policy.id)
        data_protection = VolumePatchPropertiesDataProtection(snapshot=snapshot)
        volume_patch_properties = VolumePatchProperties(data_protection=data_protection)
        volume_patch = VolumePatch(properties=volume_patch_properties)
        volume = self.client.volumes.begin_update(
            setup.TEST_RG, ACCOUNT1, setup.TEST_POOL_1, volumeName1, volume_patch
        ).result()
        wait_for_volume(self.client, setup.TEST_RG, ACCOUNT1, setup.TEST_POOL_1, volumeName1)

        assert volume.data_protection.snapshot.snapshot_policy_id == snapshot_policy.id

        # cleanup
        delete_volume(self.client, setup.TEST_RG, ACCOUNT1, setup.TEST_POOL_1, volumeName1)
        delete_snapshot_policy(self.client, setup.TEST_SNAPSHOT_POLICY_1, account_name=ACCOUNT1)
        delete_pool(self.client, setup.TEST_RG, ACCOUNT1, setup.TEST_POOL_1)
        delete_account(self.client, setup.TEST_RG, ACCOUNT1)

        print("Finished with test_assign_snapshot_policy_to_volume")



from devtools_testutils import AzureMgmtRecordedTestCase, recorded_by_proxy
import setup
import azure.mgmt.netapp.models


class TestNetAppResourceQuota(AzureMgmtRecordedTestCase):

    def setup_method(self, method):
        self.client = self.create_mgmt_client(azure.mgmt.netapp.NetAppManagementClient)

    # Before tests are run live a resource group needs to be created along with vnet and subnet
    # Note that when tests are run in live mode it is best to run one test at a time.
    @recorded_by_proxy
    def test_list_resource_quota_limit(self):
        limits = self.client.net_app_resource_quota_limits.list(setup.LOCATION)

        assert limits is not None
        assert len(list(limits)) > 0



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------
"""Customize generated code here.

Follow our quickstart for examples: https://aka.ms/azsdk/python/dpcodegen/python/customize
"""


__all__: list[str] = []  # Add all objects you want publicly available to users at this package level


def patch_sdk():
    """Do not remove from this file.

    `patch_sdk` is a last resort escape hatch that allows you to do customizations
    you can't accomplish using the techniques described in
    https://aka.ms/azsdk/python/dpcodegen/python/customize
    """



import time
from azure.mgmt.resource.resources import ResourceManagementClient
from devtools_testutils import AzureMgmtRecordedTestCase, recorded_by_proxy
from azure.mgmt.netapp.models import CapacityPool, CapacityPoolPatch, PoolPatchProperties
from test_account import create_account, delete_account
import setup
import azure.mgmt.netapp.models

DEFAULT_SIZE = 4398046511104


def create_pool(
    client,
    rg=setup.TEST_RG,
    acc_name=setup.TEST_ACC_1,
    pool_name=setup.TEST_POOL_1,
    location=setup.LOCATION,
    pool_only=False,
):
    if not pool_only:
        create_account(client, rg, acc_name, location)
    print("Creating pool {0} in NetApp Account {1}".format(pool_name, acc_name))
    pool_body = CapacityPool(service_level=setup.SERVICE_LEVEL, size=DEFAULT_SIZE, location=location)
    pool = client.pools.begin_create_or_update(rg, acc_name, pool_name, pool_body).result()
    wait_for_pool(client, rg, acc_name, pool_name)
    print("\tDone creating pool {0} in NetApp Account {1}".format(pool_name, acc_name))
    return pool


def wait_for_pool(client, rg, account_name, pool_name):
    # a work around for the async nature of certain ARM processes
    retry = 0
    while retry < 60:
        pool = client.pools.get(rg, account_name, pool_name)
        if pool.provisioning_state == "Succeeded":
            break
        if setup.LIVE:
            time.sleep(3)
        retry += 1
    if retry == 60:
        raise Exception("Timeout when waiting for pool")


def wait_for_no_pool(client, rg, acc_name, pool_name):
    # a workaround for the async nature of certain ARM processes
    retry = 0
    while retry < 100:
        try:
            client.pools.get(rg, acc_name, pool_name)
        except:
            # not found is an exception case (status code 200 expected)
            # and is actually what we are waiting for
            break
        if setup.LIVE:
            time.sleep(3)
        retry += 1
    if retry == 100:
        raise Exception("Timeout when waiting for no pool")


def delete_pool(client, rg, acc_name, pool_name):
    # nest resources seem to hang around for a little while even
    # when apparently deleted, therefore give it a chance
    print("Deleting pool {0} in NetApp Account {1}".format(pool_name, acc_name))
    retry = 0
    while retry < 3:
        try:
            client.pools.begin_delete(rg, acc_name, pool_name).wait()
            break
        except Exception as e:
            print("failed to delete pool. Retry number: {0}".format(retry))
            print(e)
            retry += 1
            if setup.LIVE:
                time.sleep(10)
    if retry == 3:
        raise Exception("Timeout when deleting pool")
    wait_for_no_pool(client, rg, acc_name, pool_name)
    print("\tDone deleting pool {0} in NetApp Account {1}".format(pool_name, acc_name))


class TestNetAppCapacityPool(AzureMgmtRecordedTestCase):

    def setup_method(self, method):
        self.client = self.create_mgmt_client(azure.mgmt.netapp.NetAppManagementClient)
        if self.is_live:
            setup.LIVE = True

    # Before tests are run live a resource group needs to be created along with vnet and subnet
    # Note that when tests are run in live mode it is best to run one test at a time.
    @recorded_by_proxy
    def test_create_delete_pool(self):
        print("Starting test_create_delete_pool")
        account_name1 = self.get_resource_name(setup.TEST_ACC_1 + "-")
        pool = create_pool(self.client, setup.TEST_RG, account_name1, setup.TEST_POOL_1, setup.LOCATION)
        assert pool.size == DEFAULT_SIZE
        assert pool.name == account_name1 + "/" + setup.TEST_POOL_1

        pool_list = self.client.pools.list(setup.TEST_RG, account_name1)
        assert len(list(pool_list)) == 1

        delete_pool(self.client, setup.TEST_RG, account_name1, setup.TEST_POOL_1)
        pool_list = self.client.pools.list(setup.TEST_RG, account_name1)
        assert len(list(pool_list)) == 0

        delete_account(self.client, setup.TEST_RG, account_name1)

        print("Finished with test_create_delete_pool")

    @recorded_by_proxy
    def test_list_pools(self):
        print("Starting test_list_pools")
        account_name1 = self.get_resource_name(setup.TEST_ACC_1 + "-")
        create_pool(self.client, setup.TEST_RG, account_name1, setup.TEST_POOL_1, setup.LOCATION)
        create_pool(self.client, setup.TEST_RG, account_name1, setup.TEST_POOL_2, setup.LOCATION, pool_only=True)
        pools = [setup.TEST_POOL_1, setup.TEST_POOL_2]

        pool_list = self.client.pools.list(setup.TEST_RG, account_name1)
        assert len(list(pool_list)) == 2
        idx = 0
        for pool in pool_list:
            assert pool.name == pools[idx]
            idx += 1

        delete_pool(self.client, setup.TEST_RG, account_name1, setup.TEST_POOL_1)
        delete_pool(self.client, setup.TEST_RG, account_name1, setup.TEST_POOL_2)
        delete_account(self.client, setup.TEST_RG, account_name1)
        print("Finished with test_list_pools")

    @recorded_by_proxy
    def test_get_pool_by_name(self):
        print("Starting test_get_pool_by_name")
        account_name1 = self.get_resource_name(setup.TEST_ACC_1 + "-")
        create_pool(self.client, setup.TEST_RG, account_name1, setup.TEST_POOL_1, setup.LOCATION)

        pool = self.client.pools.get(setup.TEST_RG, account_name1, setup.TEST_POOL_1)
        assert pool.name == account_name1 + "/" + setup.TEST_POOL_1

        delete_pool(self.client, setup.TEST_RG, account_name1, setup.TEST_POOL_1)
        delete_account(self.client, setup.TEST_RG, account_name1)
        print("Finished with test_get_pool_by_name")

    @recorded_by_proxy
    def test_update_pool(self):
        print("Starting test_update_pool")
        account_name1 = self.get_resource_name(setup.TEST_ACC_1 + "-")
        pool = create_pool(self.client, setup.TEST_RG, account_name1, setup.TEST_POOL_1)
        assert pool.qos_type == "Auto"

        pool_patchProperties = PoolPatchProperties(qos_type="Manual")
        pool_body = CapacityPoolPatch(properties=pool_patchProperties, location=setup.LOCATION)
        pool = self.client.pools.begin_create_or_update(
            setup.TEST_RG, account_name1, setup.TEST_POOL_1, pool_body
        ).result()
        assert pool.qos_type == "Manual"

        delete_pool(self.client, setup.TEST_RG, account_name1, setup.TEST_POOL_1)
        delete_account(self.client, setup.TEST_RG, account_name1)
        print("Finished with test_update_pool")

    @recorded_by_proxy
    def test_patch_pool(self):
        print("Starting test_patch_pool")
        account_name1 = self.get_resource_name(setup.TEST_ACC_1 + "-")
        create_pool(self.client, setup.TEST_RG, account_name1, setup.TEST_POOL_1)

        tag = {"Tag2": "Value1"}
        pool_patchProperties = PoolPatchProperties(qos_type="Manual")
        capacity_pool_patch = CapacityPoolPatch(properties=pool_patchProperties, tags=tag)

        print("Updating pool")
        self.client.pools.begin_update(setup.TEST_RG, account_name1, setup.TEST_POOL_1, capacity_pool_patch).result()
        print("Done updating pool")
        pool = self.client.pools.get(setup.TEST_RG, account_name1, setup.TEST_POOL_1)
        assert pool.qos_type == "Manual"
        assert pool.tags["Tag2"] == "Value1"

        delete_pool(self.client, setup.TEST_RG, account_name1, setup.TEST_POOL_1)
        delete_account(self.client, setup.TEST_RG, account_name1)
        print("Finished with test_patch_pool")
